import calcdiagram_functions as calc
import logconfig as log
from shared import safeparse as sp

myLogger = log.antSysDiagLogger.logger

class ImportedProtcolsData():
    def _init_(self):
        self.protocolDataList=[]

class ProtocolData():
    def _init_(self):
        self.protocolPath=''
        self.ptu2DataList=[]
        self.txAntDiagDataList=[]

class PTU2Data():
    def _init_(self):
        self.filesList=[]
        self.protocolPath=''


class TxAntDiagData():
    def _init_(self):
        self.filesList=[]
        self.protocolPath=''


class ProtocolsTableData():
    def __init__(self,protocolPath,sysDiagFilePathList,txAntDiagFilePathList):
        self.protocolPath=protocolPath
        self.sysDiagFilePathShapedList=[]
        self.txAntDiagFilePathShapedList=[]
        self.biggerListLength=0
        self.smallerListLength=0
        self.smallerListColIdx=0
        self.smallerListRowSpanList=[]


        if len(sysDiagFilePathList) >= len(txAntDiagFilePathList):
            if len(txAntDiagFilePathList)!=0:
                ratio = int(len(sysDiagFilePathList) / len(txAntDiagFilePathList))
                res=len(sysDiagFilePathList) % len(txAntDiagFilePathList)

                self.smallerListColIdx=3
                self.biggerListLength=len(sysDiagFilePathList)
                self.smallerListLength=len(txAntDiagFilePathList)

                self.sysDiagFilePathShapedList=sysDiagFilePathList

                for sysDiagIdx in range(len(sysDiagFilePathList)):


                    entry=''

                    for txAntDiagIdx in range(len(txAntDiagFilePathList)):

                        if ratio * txAntDiagIdx ==sysDiagIdx:
                            entry=txAntDiagFilePathList[txAntDiagIdx]

                    self.txAntDiagFilePathShapedList.append(entry)

                size=0

                for txAntDiagIdx in range(len(txAntDiagFilePathList)):

                    tempDict = {
                        'spanStart': 0,
                        'spanCount': 0
                    }

                    tempDict['spanStart']= txAntDiagIdx * ratio



                    if txAntDiagIdx ==len(txAntDiagFilePathList)-1:
                        if res != 0:
                            tempDict['spanCount']=self.biggerListLength - size
                        else:
                            tempDict['spanCount']=ratio

                    else:
                        tempDict['spanCount']=ratio
                        size=ratio* (txAntDiagIdx +1)

                    self.smallerListRowSpanList.append(tempDict)

            else:

                self.smallerListColIdx = 3
                self.biggerListLength = len(sysDiagFilePathList)

                self.sysDiagFilePathShapedList = sysDiagFilePathList

                for sysDiagIdx in range(len(sysDiagFilePathList)):

                    entry = ''

                    self.txAntDiagFilePathShapedList.append(entry)


                tempDict = {
                    'spanStart': 0,
                    'spanCount': self.biggerListLength
                }

                self.smallerListRowSpanList.append(tempDict)
        else:
            if len(sysDiagFilePathList)!=0:

                ratio = int(len(txAntDiagFilePathList) / len(sysDiagFilePathList))
                res=len(txAntDiagFilePathList) % len(sysDiagFilePathList)

                self.smallerListColIdx=2
                self.biggerListLength=len(txAntDiagFilePathList)
                self.smallerListLength = len(sysDiagFilePathList)

                self.txAntDiagFilePathShapedList=txAntDiagFilePathList

                for txAntDiagIdx in range(len(txAntDiagFilePathList)):

                    entry=''

                    for sysDiagIdx in range(len(sysDiagFilePathList)):

                        if ratio * sysDiagIdx ==txAntDiagIdx:
                            entry = sysDiagFilePathList[sysDiagIdx]

                    self.sysDiagFilePathShapedList.append(entry)

                size=0
                for sysDiagIdx in range(len(sysDiagFilePathList)):
                    tempDict={
                        'spanStart':0,
                        'spanCount':0
                    }

                    tempDict['spanStart']=sysDiagIdx * ratio

                    if sysDiagIdx ==len(sysDiagFilePathList)-1:
                        if res != 0:
                            tempDict['spanCount'] =self.biggerListLength - size
                        else:
                            tempDict['spanCount'] = ratio
                    else:
                        tempDict['spanCount']=ratio
                        size = ratio * (sysDiagIdx +1)

                    self.smallerListRowSpanList.append(tempDict)
            else:

                self.smallerListColIdx = 2
                self.biggerListLength = len(txAntDiagFilePathList)

                self.txAntDiagFilePathShapedList = txAntDiagFilePathList

                for txAntDiagIdx in range(len(txAntDiagFilePathList)):

                    entry = ''

                    self.sysDiagFilePathShapedList.append(entry)


                tempDict = {
                    'spanStart': 0,
                    'spanCount': self.biggerListLength
                }

                self.smallerListRowSpanList.append(tempDict)


def parsePlotWindowString(plotWindowStr):
    plotWindowConfig = {

        "graph": "",
        "antType": 0,
        "diagramNum": 0,
        "rxCh": 0
    }

    graphConfig = [ "level","avgLevel","normLevel","avgNormLevel","phaseDiff","avgPhaseDiff","diffPhaseDiff","avgDiffPhaseDiff","corrPhaseDiff"]


    graphFound =0
    antTypeFound=0
    diagramNumFound = 0
    rxChFound = 0

    plotWindowStr = plotWindowStr.replace('.','_')
    nameParts = plotWindowStr.split('_')

    for part in nameParts:
        if graphFound == 0:
               if part in graphConfig:
                   plotWindowConfig["graph"]=part
                   graphFound = 1
        if antTypeFound == 0:
            if part.startswith('AntType'):
                part = part.strip('AntType')
                if (sp.checkInt(part)):
                    plotWindowConfig["antType"] = int(part)
                    antTypeFound = 1
        if diagramNumFound == 0:
            if part.startswith('Diagram'):
                part = part.strip('Diagram')
                if (sp.checkInt(part)):
                    plotWindowConfig["diagramNum"] = int(part)
                    diagramNumFound = 1
        if rxChFound == 0:
            if part.startswith('RxCh'):
                part = part.strip('RxCh')
                if (sp.checkInt(part)):
                    plotWindowConfig["rxCh"] = int(part)
                    rxChFound = 1

    return plotWindowConfig

def rxChStringMapping(graphTitle, rxCh):
    rxChString=''
    # Mapping string for the phase difference channels
    # phDiffRxComb = ['RxAnt1-RxAnt0', 'RxAnt2-RxAnt1', 'RxAnt3-RxAnt2']
    if 'haseDiff'  in graphTitle:
        lower=str(rxCh)
        upper=str(rxCh+1)
        rxChString='RxAnt'+upper+'-'+'RxAnt'+lower
    else:
        rxChString='RxAnt'+ str(rxCh)
    return rxChString

def graphTypeStringMapping(graphTitle):
    graphTypeString = ''
    if 'haseDiff' in graphTitle:
        graphTypeString = "all sensors, RxAnt comb.:"
    else:
        graphTypeString = "all sensors, RxAnt:"
    return graphTypeString

# Mapping string for the antenna diagram types
def getAntDiagType(inputString):
    if inputString == 'Az':
        antennaDiagType = 'Azimuth'
    else:
        antennaDiagType = 'Elevation'
    return antennaDiagType

# Get the num. of diagrams from the parsed data
def getNofDiagrams(antType,parseDataList):
    diagramIdxList=[]
    for pdIdx in range(len(parseDataList)):
        if parseDataList[pdIdx].antennaInfo.antennaType==antType:
            diagram=parseDataList[pdIdx].fileTitle.diagram
            diagramIdxList.append(diagram)
    if len(diagramIdxList)!=0:
        nofDiagrams=max(diagramIdxList)+1
    else:
        nofDiagrams=0
    return nofDiagrams

def getDiagramList(antType,parseDataList):
    diagramIdxList=[]
    for pdIdx in range(len(parseDataList)):
        if parseDataList[pdIdx].antennaInfo.antennaType==antType:
            diagram=parseDataList[pdIdx].fileTitle.diagram
            diagramIdxList.append(diagram)

    # Preserve original order while removing duplicates
    outputList=[]
    for diagram in diagramIdxList:
        if diagram not in outputList:
            outputList.append(diagram)

    return outputList

def getDateAndTimeList(parseDataList):
    dateAndTime = ''
    dateAndTimeList = []
    for listIdx in range(len(parseDataList)):
        if parseDataList[listIdx].fileTitle.dateAndTimeFound != 0:
            if parseDataList[listIdx].fileTitle.sensor!= dateAndTime:
                dateAndTime = parseDataList[listIdx].fileTitle.dateAndTime
                dateAndTimeList.append(parseDataList[listIdx].fileTitle.dateAndTime)
    return dateAndTimeList

# Get the list of all sensors from the parsed data
def getSensorList(parseDataList):
    sensor = ''
    sensorList = []
    for listIdx in range(len(parseDataList)):
        if parseDataList[listIdx].fileTitle.sensorFound != 0:
            if parseDataList[listIdx].fileTitle.sensor!= sensor:
                sensor = parseDataList[listIdx].fileTitle.sensor
                sensorList.append(parseDataList[listIdx].fileTitle.sensor)
    return sensorList

def getDateTimeSensorCombList(antennaType,parseDataList):
    outputDictList=[]
    sensor=''
    dateAndTime=''
    for listIdx in range(len(parseDataList)):
        outputDict = dict()

        if parseDataList[listIdx].fileTitle.sensorFound != 0 and parseDataList[listIdx].fileTitle.dateAndTimeFound !=0:
            if parseDataList[listIdx].fileTitle.sensor != sensor or parseDataList[listIdx].fileTitle.dateAndTime != dateAndTime:
                dateAndTime= parseDataList[listIdx].fileTitle.dateAndTime
                sensor= parseDataList[listIdx].fileTitle.sensor
                outputDict.update({"dateAndTime":dateAndTime,"sensor":sensor})
                outputDictList.append(outputDict)
    return outputDictList

def getDateTimeSensorCombTable(dateTimeList,sensorList,dateAndTimeCombList):

    outputList=[]
    for k in range(len(dateAndTimeCombList)):
        for i in range(len(dateTimeList)):
            if dateAndTimeCombList[k]["dateAndTime"] == dateTimeList[i]:
                subList = []
                for j in range(len(sensorList)):
                    if dateAndTimeCombList[k]["sensor"]==sensorList[j]:
                        subList.append(sensorList[j])
            outputList.append(subList)

    return outputList

#



#Get number of RxAnt per sensor
def getNofRxAntPerSensor(sensor, parseDataList):
    actualNofRxAnt=0
    for pdIdx in range(len(parseDataList)):
        if parseDataList[pdIdx].antennaInfo.fileTitle.sensorFound==1:
            if sensor == parseDataList[pdIdx].fileTitle.sensor:
                actualNofRxAnt = parseDataList[pdIdx].antennaInfo.nofAntennas
                break
    return actualNofRxAnt

# Get the maximal number of Tx antenas of all sensors
def getMaxNofTxAnt(sensorList,parseDataList):
    maxNofTxAnt=0
    maxNofTxAntList=[]
    for sensorIdx in range(len(sensorList)):
        maxNofTxAnt=getNofTxAntPerSensor(sensorList[sensorIdx],parseDataList)
        maxNofTxAntList.append(maxNofTxAnt)
    return max(maxNofTxAntList)

# Get number of TxAnt per sensor
def getNofTxAntPerSensor(sensor, parseDataList):
    actualTxAnt=0
    txAntList=[]

    for pdIdx in range(len(parseDataList)):
        if parseDataList[pdIdx].antennaInfo.fileTitle.sensorFound==1:
            if sensor == parseDataList[pdIdx].fileTitle.sensor:
                if parseDataList[pdIdx].antennaInfo.fileTitle.txAntFound == 1:
                    actualTxAnt = parseDataList[pdIdx].fileTitle.txAnt
                    txAntList.append(actualTxAnt)
    return max(txAntList)

# Get TxAnt list
def getTxAntList(parseDataList):
    actualTxAnt=0
    txAntList=[]
    for pdIdx in range(len(parseDataList)):
        if parseDataList[pdIdx].antennaInfo.fileTitle.txAntFound == 1:
            actualTxAnt = parseDataList[pdIdx].fileTitle.txAnt
            txAntList.append(actualTxAnt)

    # Preserve original order while removing duplicates
    outputList=[]
    for txAnt in txAntList:
        if txAnt not in outputList:
            outputList.append(txAnt)

    return outputList



def getAngleStep(parseData,isTargetAngle,upsideDown):
    if isTargetAngle ==1:
        angle=calc.getTarAngVals(parseData,upsideDown)
    else:
        angle=calc.getPtuAngVals(parseData)
    firstPoint=round(angle[0],1)
    lastPoint=round(angle[1],1)
    step=abs(firstPoint-lastPoint)
    return step

def getNofAnglePoints(parseData,isTargetAngle,upsideDown):
    if isTargetAngle == 1:
        length=len(calc.getTarAngVals(parseData, upsideDown))
    else:
        length=len(calc.getPtuAngVals(parseData))
    return length

def getRefAnglePoint(parseData,isTargetAngle,upsideDown):
    if isTargetAngle ==1:
        angle=calc.getTarAngVals(parseData,upsideDown)
    else:
        angle=calc.getPtuAngVals(parseData)
    refAnglePoint=round(angle[0],1)
    return refAnglePoint


# Get the desired diagrams list, according to the diagram configuration.
def getDesiredDiagramsList(diagramConfig, diagramConfigList,antType, parseDataList):
    print(f"[DEBUG] getDesiredDiagramsList called with:")
    print(f"  - antType: {antType}")
    print(f"  - parseDataList length: {len(parseDataList)}")
    print(f"  - diagramConfig: {diagramConfig}")
    print(f"  - diagramConfigList: {diagramConfigList}")
    
    desiredDiagramList=[]
    diagramTempList=getDiagramList(antType,parseDataList)
    print(f"  - diagramTempList from getDiagramList: {diagramTempList}")
    
    # nofDiagrams = getNofDiagrams(antType,parseDataList)
    high = diagramConfig["higherNumber"]
    low = diagramConfig["lowerNumber"]

    if diagramConfig['selectDiagrams']==0:
        if diagramConfig['oddOrEven']==0:
            desiredDiagramList=diagramTempList
            # for diagramIdx in range(nofDiagrams):
            #     desiredDiagramList.append(diagramIdx)
        elif diagramConfig['oddOrEven'] == 1:
            for i in range(len(diagramTempList)):
                if diagramTempList[i] % 2 !=0:
                    desiredDiagramList.append(diagramTempList[i])

            # for i in range(1, nofDiagrams, 2):
            #     desiredDiagramList.append(i)
        else:
            for i in range(len(diagramTempList)):
                if diagramTempList[i] % 2 == 0:
                    desiredDiagramList.append(diagramTempList[i])

            # for i in range(0, nofDiagrams, 2):
            #     desiredDiagramList.append(i)

    elif diagramConfig['selectDiagrams']==1:
        if diagramConfig['oddOrEven']==0:
            for diagIdx in range (len(diagramTempList)):
                if diagramTempList[diagIdx]>=low and diagramTempList[diagIdx]<=high:
                    desiredDiagramList.append(diagramTempList[diagIdx])
            # for diagramIdx in range(low,high+1,1):
            #     desiredDiagramList.append(diagramIdx)
        elif diagramConfig['oddOrEven'] == 1:
            for diagIdx in range (len(diagramTempList)):
                if diagramTempList[diagIdx]>=low and diagramTempList[diagIdx]<=high and diagramTempList[diagIdx]%2!=0:
                    desiredDiagramList.append(diagramTempList[diagIdx])
            # if (low % 2) == 0:
            #     low=low+1
            # else:
            #     low=low
            # for i in range(low, high+1, 2):
            #     desiredDiagramList.append(i)
        else:


            for diagIdx in range (len(diagramTempList)):
                if diagramTempList[diagIdx]>=low and diagramTempList[diagIdx]<=high and diagramTempList[diagIdx]%2==0:
                    desiredDiagramList.append(diagramTempList[diagIdx])
            # if (low % 2) == 0:
            #     low = low
            # else:
            #     low = low+1
            # for i in range(low, high+1, 2):
            #     desiredDiagramList.append(i)
    else:
        # Preserve original order while removing duplicates
        filteredDiagList = []
        for diagram in diagramConfigList:
            if diagram not in filteredDiagList:
                filteredDiagList.append(diagram)

            for diagTempIdx in range(len(diagramTempList)):
                for filtDiagIdx in range(len(filteredDiagList)):
                    if diagramTempList[diagTempIdx] == filteredDiagList[filtDiagIdx]:
                        desiredDiagramList.append(diagramTempList[diagTempIdx])

    print(f"  - Final desiredDiagramList: {desiredDiagramList}")
    return desiredDiagramList

# Returns a list of all parsed PTU2 files for a certain diagram.
def getDataPerDiagram(desiredDiagram,loadedData):
    outputList=[]
    for pdIdx in range(len(loadedData.parseDataList)):
        if desiredDiagram==loadedData.parseDataList[pdIdx].fileTitle.diagram:
            outputList.append(loadedData.parseDataList[pdIdx])
    return outputList


# Get the configured chirps
def getChirpList(chirpSetup):
    chirpList = []
    for diagram, set in chirpSetup.items():
        if set == 1:
            chirpList.append(diagram)
    return chirpList

# Get the valid chirp according to the configuration and the active chirps info from the PTU2 files.
def getValidChirp( chirpSetup, parseDataList, pdIdx):
    chirpList = getChirpList(chirpSetup)
    validChirpList = []
    for chirpIdx in range(len(chirpList)):
        if parseDataList[pdIdx].chirpInfoList[chirpIdx].active == 1:
            validChirpList.append(chirpList[chirpIdx])
    return validChirpList


# Get the actual number of Rx channels, per specific graph and a PTU2 file.
def getNofActualRxChannels(graphList, graphIdx, parseDataList, parseDataIdx):
    if graphList[graphIdx].endswith('evel'):
        actualNofRxCh = parseDataList[parseDataIdx].antennaInfo.nofAntennas
    else:
        actualNofRxCh = parseDataList[parseDataIdx].antennaInfo.nofAntennaDiffValues
    return actualNofRxCh

# Get the actual number of Rx channels, per specific graph and a PTU2 file.
def getNofActualRxChannels(graph, parseDataList, pdIdx):
    if graph.endswith('evel'):
        actualNofRxCh =parseDataList[pdIdx].antennaInfo.nofAntennas
    else:
        actualNofRxCh = parseDataList[pdIdx].antennaInfo.nofAntennaDiffValues
    return actualNofRxCh

# Get the maximal number of Rx channels for specific graph, of all PTU2 files.
def getMaxNofRxCh(graph,parseDataList):
    nofRxChList = []
    for pdIdx in range(len(parseDataList)):

        nofRxCh = getNofActualRxChannels(graph, parseDataList, pdIdx)
        nofRxChList.append(nofRxCh)
    return max(nofRxChList)

# Get the data for the actual graph for specific graph, chirp, rx channel, and PTU2 file
def getActualGraphData(graph, rxChannel, chirp, parseDataList, pdIdx):
    actualDiagramData = 0
    if graph == "level":
        actualDiagramData = calc.getAntLevel(rxChannel, chirp, parseDataList[pdIdx])

    elif graph == "avgLevel":  # Average power level of chirps [dB].
        actualDiagramData = calc.getAvgAntLevel(rxChannel, parseDataList[pdIdx])

    elif graph == "normLevel":  # Normalised power level [dB].
        actualDiagramData = calc.getNormAntLevel(rxChannel, chirp, parseDataList[pdIdx])

    elif graph == "avgNormLevel":  # Average of normalised power levels of all chirps [dB].
        actualDiagramData = calc.getAvgNormAntLevel(rxChannel, parseDataList[pdIdx])

    # Phase difference diagrams
    elif graph == "phaseDiff":  # Phase difference [°].
        actualDiagramData = calc.getPhaseDiff(rxChannel, chirp, parseDataList[pdIdx])

    elif graph == "avgPhaseDiff":  # Average of the phase differences of all chirps[°].
        actualDiagramData = calc.getAvgPhaseDiff(rxChannel, parseDataList[pdIdx])

    elif graph == "idealPhaseDiff":  # Ideal phase difference [°].
        actualDiagramData = calc.getIdealPhaseDiff(rxChannel, chirp, parseDataList[pdIdx])

    elif graph == "diffPhaseDiff":  # Difference between one phase difference and the ideal one [°].
        actualDiagramData = calc.getPhaseDiffDiff(rxChannel, chirp, parseDataList[pdIdx])

    elif graph == "avgDiffPhaseDiff":  # Average difference of between the phase differences and the ideal one of all chirps[°].
        actualDiagramData = calc.getAvgPhaseDiffDiff(rxChannel, parseDataList[pdIdx])

    elif graph == "corrPhaseDiff":
        actualDiagramData = calc.getPhaseDiffOffCorr(rxChannel, chirp, parseDataList[pdIdx])
    return actualDiagramData


# def scaleLowerMargin(graph, chirpSetup, parseDataList):
#     actualGraph = 0
#     lowerMarginList = []
#     for pdIdx in range(len(parseDataList)):
#         validChirpList = getValidChirp(chirpSetup, parseDataList[pdIdx])
#         nofRxCh = getMaxNofRxCh(graph, parseDataList)
#         for chirp in range(len(validChirpList)):
#             for rxCh in range(nofRxCh):
#                 actualGraph = getActualGraphData(graph, rxCh, chirp, parseDataList[pdIdx])
#                 minValue = actualGraph.min()
#                 lowerMarginList.append(minValue)
#
#     lowerMargin = min(lowerMarginList)
#     whole = int(abs(lowerMargin) / 10)
#     rest = abs(lowerMargin) % 10
#
#     if lowerMargin >= 0:
#         if rest > 5:
#             lowerMargin = whole * 10
#         else:
#             lowerMargin = whole * 10 - 10
#     else:
#         if rest < 5:
#             lowerMargin = whole * (-10) - 10
#         else:
#             lowerMargin = whole * (-10) - 20
#     return lowerMargin
#
#     # Upper margin of all drawn sensor diagrams for a specific graph
#
#
# def scaleUpperMargin(graph, chirpSetup, parseDataList):
#     actualGraph = 0
#     upperMarginList = []
#     for pdIdx in range(len(parseDataList)):
#         validChirpList = getValidChirp(chirpSetup, parseDataList[pdIdx])
#         nofRxCh = getMaxNofRxCh(graph, parseDataList)
#         for chirp in range(len(validChirpList)):
#             for rxCh in range(nofRxCh):
#                 actualGraph = getActualGraphData(graph, rxCh, chirp,parseDataList[pdIdx])
#                 maxValue = actualGraph.max()
#                 upperMarginList.append(maxValue)
#     upperMargin = max(upperMarginList)
#     whole = int(abs(upperMargin) / 10)
#     rest = abs(upperMargin) % 10
#
#     if upperMargin >= 0:
#         if rest > 5:
#             upperMargin = whole * 10 + 20
#         else:
#             upperMargin = whole * 10 + 10
#     else:
#         if rest < 5:
#             upperMargin = whole * (-10) + 10
#         else:
#             upperMargin = whole * (-10)
#     return upperMargin

def azDataFound(parseDataList):
    val=False
    for pdIdx in range(len(parseDataList)):
        if (parseDataList[pdIdx].measSetupInfo.antennaDiagramTypeFound == 1) and (parseDataList[pdIdx].measSetupInfo.antennaDiagramType == 'Az'):
            val=True
            break
    return val

def noDiagTypeDataFound(parseDataList):
    val=False
    for pdIdx in range(len(parseDataList)):
        if (parseDataList[pdIdx].measSetupInfo.antennaDiagramTypeFound == 0):
            val=True
            break
    return val

# Get the list of all antenna types from the parsed data
def getAntennaTypesList(parseDataList):
    antennaType = 0
    antennaTypeList = []

    for listIdx in range(len(parseDataList)):

        if parseDataList[listIdx].antennaInfo.antennaType!= antennaType:
            antennaType = parseDataList[listIdx].antennaInfo.antennaType
            antennaTypeList.append(parseDataList[listIdx].antennaInfo.antennaType)

    antTypeSet=set(antennaTypeList)
    outputList=list(antTypeSet)
    return outputList

def getPtu2ListPerAntennaType(antType,parseDataList):
    outputList=[]
    for listIdx in range(len(parseDataList)):
        if parseDataList[listIdx].antennaInfo.antennaType == antType:
            outputList.append(parseDataList[listIdx])
    return outputList

def getFreqListPerDiagram(chirpConfig,diagNum,parseDataList):
    freqList=[]
    for pdIdx in range(len(parseDataList)):
        if parseDataList[pdIdx].fileTitle.diagramFound==1:
            if parseDataList[pdIdx].fileTitle.diagram == diagNum:
                validChirpList = getValidChirp(chirpConfig, parseDataList, pdIdx)
                for chirp in range(len(validChirpList)):
                    freqList.append(parseDataList[pdIdx].chirpInfoList[chirp].centerFrequency)

    return  freqList

def getFreqList(chirpConfig, diagramList,parseDataList):
    freqList=[]
    for pdIdx in range(len(parseDataList)):
        for diagIdx in range(len(diagramList)):
            if parseDataList[pdIdx].fileTitle.diagramFound==1:
                validChirpList = getValidChirp(chirpConfig, parseDataList, pdIdx)
                for chirp in range(len(validChirpList)):
                    freqList.append(parseDataList[pdIdx].chirpInfoList[chirp].centerFrequency)

    return  freqList

def getAvgFreqPerDiagram (chirpConfig,diagNum,parseDataList):
    freqList=getFreqListPerDiagram(chirpConfig,diagNum,parseDataList)

    avgFreq = 0
    if len(freqList)!=0:
        avgFreq=sum(freqList)/len(freqList)
    else:
        myLogger.error("Empty frequency list.")
    return avgFreq


def getMaxFreqPerDiagram (chirpConfig,diagNum,parseDataList):
    freqList=getFreqListPerDiagram(chirpConfig,diagNum,parseDataList)
    maxFreq = 0
    if len(freqList)!=0:
        maxFreq=max(freqList)
    else:
        myLogger.error("Empty frequency list.")
    return maxFreq

def getMinFreqPerDiagram (chirpConfig,diagNum,parseDataList):
    freqList=getFreqListPerDiagram(chirpConfig,diagNum,parseDataList)

    minFreq = 0
    if len(freqList)!=0:
        minFreq=min(freqList)
    else:
        myLogger.error("Empty frequency list.")
    return minFreq

def getAvgFreq (chirpConfig,diagramList,parseDataList):
    freqList=getFreqList(chirpConfig,diagramList,parseDataList)

    avgFreq = 0
    if len(freqList)!=0:
        avgFreq=sum(freqList)/len(freqList)
    else:
        myLogger.error("Empty frequency list.")
    return avgFreq


def getMaxFreq(chirpConfig,diagramList,parseDataList):
    freqList=getFreqList(chirpConfig,diagramList,parseDataList)
    maxFreq = 0
    if len(freqList)!=0:
        maxFreq=max(freqList)
    else:
        myLogger.error("Empty frequency list.")
    return maxFreq

def getMinFreq (chirpConfig,diagramList,parseDataList):
    freqList=getFreqList(chirpConfig,diagramList,parseDataList)

    minFreq = 0
    if len(freqList)!=0:
        minFreq=min(freqList)
    else:
        myLogger.error("Empty frequency list.")
    return minFreq



import os
from enum import Enum
import io
from shared import safeparse as sp
import logconfig as log

myLogger=log.mainLoadDataLogger.logger

## Parse data module ##

# PTU angle and chirp measurement data per angle
class PtuPosMeasData:
    def __init__(self):
        self.d_PTU_AzAngle = -10000 # [°]
        self.d_PTU_ElAngle = -10000 # [°]
        self.level_dBm = -10000 #[dBm]
        self.levelPowMet_dBm =-10000 #[dBm]
        self.frequenz_MHz = 0 #[MHz]
        self.time_ms = 0 #[ms]
        self.levelMeasSpec_dBm = -10000 #[dBm]
        self.levelMeasPowMet_dBm = -10000 #[dBm]


# Data from the parsed TxAnt diagram file, containing meaurements data for each ptu position
class TxAntDiagramParseData:

    def __init__(self):
        self.bDataValid = False
        self.fileNamePath = ''
        self.errorList = []
        self.ptuMaxSpecHeaderData = 0
        self.chirp = 0
        self.ptuPosDataList = [] # Messdaten an der jeweiligen ptu position
        self.fileTitle = FileTitle()
    # Call parse data from the PTU2ParseData object (parseData wrapper)
    def parseFile(self, fileNamePath, useSensorMuster, sensorMusterList):
        self.errorList = []
        self.fileNamePath = fileNamePath
        parseFile(fileNamePath, self)
        self.fileTitle.parseFileTitle(self.fileNamePath, useSensorMuster, sensorMusterList)


# Meta data class from the PTU2 file title
class FileTitle:
    def __init__(self):
        self.dateAndTime  = ''
        self.date    = ''
        self.time    = ''
        self.sensor  = ''
        self.tuner   = ''
        self.txAnt   = 0
        self.seqNum  = 0
        self.diagram = 0
        self.rf      = ''
        self.dsp     = ''
        self.fileNamePath = ''
        self.fileDirPath  = ''
        self.fileName = ''
        self.sensorMusterList = []
        self.useSensorMuster = 0
        self.sensorMusterValueDict = dict()

        self.dateAndTimeFound = 0
        self.tunerFound   = 0
        self.rfFound      = 0
        self.dspFound     = 0
        self.txAntFound   = 0
        self.seqNumFound  = 0
        self.diagramFound = 0
        self.sensorFound  = 0
        self.sensorMusterFoundDict=dict()

    # Parse the PTU2 file title
    def parseFileTitle(self, fileNamePath, useSensorMuster, sensorMusterList):

        self.fileNamePath     = fileNamePath
        self.useSensorMuster  = useSensorMuster
        self.sensorMusterList = sensorMusterList

        if useSensorMuster == 1:
            if len(sensorMusterList) != 0:
                for musterItem in sensorMusterList:
                    dictKey = str(musterItem)
                    self.sensorMusterFoundDict[dictKey] = 0

                    self.sensorMusterValueDict[dictKey] = ''
            else:
                myLogger.error('Empty sensor muster list!')

        pathParts = os.path.split(fileNamePath)

        fileName = pathParts[1]
        self.fileName = fileName

        fileDirPath = pathParts[0]
        self.fileDirPath = fileDirPath


        fileNameParts = os.path.splitext(fileName)
        fileNameParts = fileNameParts[0].split('_')

        for part in fileNameParts:
            if self.dateAndTimeFound == 0:
                if part.startswith('20') and part[4] == '-':
                    self.dateAndTime = part
                    self.dateAndTimeFound = 1
                    self.date = part[0:10]
                    self.time = part[11:19]
            if self.tunerFound == 0:
                if part.startswith('Tuner'):
                    self.tuner = part
                    self.tunerFound = 1
            if self.rfFound == 0:
                if part.startswith('rf'):
                    self.rf = part
                    self.rfFound = 1
            if self.dspFound == 0:
                if part.startswith('dsp'):
                    self.dsp = part
                    self.dspFound = 1
            if self.seqNumFound == 0:
                if 'Seq' in part:
                    # self.seqNumFound = 1
                    part = part.strip('Seq')
                    if 'TxAnt' not in part:
                        if (sp.checkInt(part)):
                            self.seqNum = int(part)
                    else:
                        a=part.replace('TxAnt','.')

                        if (sp.checkInt(a[0])):
                            self.seqNum = int(a[0])
                            self.seqNumFound = 1
                        else:
                            self.seqNumFound = 0
                        if (sp.checkInt(a[2])):
                            self.txAnt = int(a[2])
                            self.txAntFound = 1
                        else:
                            self.txAntFound = 0
            if self.txAntFound == 0:
                if part.startswith('TxAnt'):
                    part = part.strip('TxAnt')
                    if(sp.checkInt(part)):
                        self.txAnt = int(part)
                        self.txAntFound = 1
            if self.diagramFound == 0:
                if part.startswith('Diagram'):
                    part = part.strip('Diagram')
                    if (sp.checkInt(part)):
                        self.diagram = int(part)
                        self.diagramFound = 1

            if self.useSensorMuster == 1:
                if len(self.sensorMusterList) != 0:
                    for musterItem in self.sensorMusterList:
                        musterItem = str(musterItem)
                        if self.sensorMusterFoundDict[musterItem] == 0:
                            if part.startswith(musterItem):
                                self.sensorMusterValueDict[musterItem] = part
                                self.sensorMusterFoundDict[musterItem] = 1

        if self.sensorFound == 0:
            if self.useSensorMuster == 1:

                if len(self.sensorMusterList) != 0:
                    i = 0
                    self.sensorFound = 1
                    for key,val in self.sensorMusterFoundDict.items():
                        if self.sensorMusterFoundDict[key] == 1:
                            binder = '' if i == 0 else '_'
                            self.sensor += binder+ self.sensorMusterValueDict[key]
                            i += 1
                        else:
                            self.sensorFound = 0

            else:
                if self.rfFound == 1:
                    binder = '_' if self.dspFound == 1 else ''
                    self.sensor = self.rf + binder + self.dsp
                    self.sensorFound = 1

def setParDict(lineArray, nofMeasDataFields):

        parMappingDict = dict()
        dictEntry = {}

        for lineArrayElement in range(nofMeasDataFields):

            if 'PTU-AngleAz' in lineArray[lineArrayElement]:
                dictEntry = {lineArrayElement: 'ptuAngleAz'}

            elif 'PTU-AngleEl' in lineArray[lineArrayElement]:
                dictEntry = {lineArrayElement: 'ptuAngleEl'}

            elif 'Level[dBm]' == lineArray[lineArrayElement]:
                dictEntry = {lineArrayElement: 'level'}

            elif ('Powermeter' in lineArray[lineArrayElement]) and ('measured' not in lineArray[lineArrayElement]):
                dictEntry = {lineArrayElement: 'levelPowerMeter'}

            elif ('Spec' in lineArray[lineArrayElement]) and ('measured' in lineArray[lineArrayElement]):
                dictEntry = {lineArrayElement: 'levelSpecMeas'}

            elif ('Powermeter' in lineArray[lineArrayElement]) and ('measured' in lineArray[lineArrayElement]):
                dictEntry = {lineArrayElement: 'levelPowerMeterMeas'}

            elif 'Frequenz' in lineArray[lineArrayElement]:
                dictEntry = {lineArrayElement: 'frequency'}

            elif 'Time' in lineArray[lineArrayElement]:
                dictEntry = {lineArrayElement: 'time'}

            parMappingDict.update(dictEntry)

        return parMappingDict


# PTU2 file parts procesing info
class searchParts(Enum):
    MaxHeader     = 1
    MaxHeaderData = 2
    Header        = 3
    PtuPosData    = 4



# Control wrapper for interpretData
def parseFile(fileNamePath, txAntDiagParseData):
    myLogger.info("Parsing: " + fileNamePath)
    txAntDiagParseData.fileNamePath = fileNamePath
    interpretData(fileNamePath, txAntDiagParseData)
    if len(txAntDiagParseData.errorList) == 0:
        txAntDiagParseData.bDataValid = True
    else:
        txAntDiagParseData.bDataValid = False
    return txAntDiagParseData

# Read and interpret the data for every part of the PTU2 file
def interpretData(fileNamePath, txAntDiagParseData):
    strMeasName = txAntDiagParseData.fileNamePath

    headerLineIndicator0 = "% PTU-Angle"
    headerLineIndicator1 = "Time[ms]"

    headerFound = 0
    ptuPosDataFound = 0
    maxHeaderFound = 0

    nofMeasDataFields = 0
    nofActualMeasDataFields = 0

    parMappingDict = dict()

    endOfEmptyLines = 0
    emptyCharFound  = 0

    currentPart = searchParts.MaxHeader

    # Try to read the file
    try:
        f = io.open(fileNamePath)

    # Catch an exception
    except IOError as ioErr:

        # errorString="try reading: " +fileNamePath+ " Exception = " + str(ioErr)
        errorString = strMeasName + ' File does not exist! Insert a valid file path.'

        txAntDiagParseData.errorList.append(errorString)
    else:
        # Read the lines of the file and remove all white spaces
        with f:
            content = f.readlines()
        content = [line.strip() for line in content]

        # Traverse through every line of the content
        for line in content:
            # read file name from protocol
            if 'File' in line:
                rest = line.split(';')
                txAntDiagParseData.fileNamePath = rest[1]
            
            timeFound = 0

            # Header lookup
            lineArray = line.split(';')


            for idx in range(len(lineArray)):

                if headerLineIndicator1 in lineArray[idx]:
                    timeFound = 1

            if currentPart == searchParts.MaxHeader:
                if ((headerLineIndicator0 in lineArray[0]) and (timeFound != 1)):
                    maxHeaderFound += 1
                    lineArrayLength = len(lineArray)
                    if lineArray[lineArrayLength - 1] == '':

                        nofMeasDataFields = len(lineArray) - 1
                    else:

                        nofMeasDataFields = len(lineArray)

                    parMappingDict = setParDict(lineArray, nofMeasDataFields)

                    currentPart = searchParts.MaxHeaderData

            elif currentPart == searchParts.MaxHeaderData:
                if not ((len(lineArray) == 1) and (lineArray[0] == '')):
                    endOfEmptyLines = 1
                if endOfEmptyLines == 1:

                    lineArrayLength = len(lineArray)

                    if lineArray[lineArrayLength - 1] == '':

                        nofActualMeasDataFields = len(lineArray) - 1
                    else:
                        nofActualMeasDataFields = len(lineArray)

                    if nofActualMeasDataFields != nofMeasDataFields:
                        txAntDiagParseData.errorList.append(strMeasName + ' incomplete or invalid ptu MAXIMUM measurement data')
                        return
                    else:
                        if not readPosPtuData(lineArray, txAntDiagParseData, nofMeasDataFields, parMappingDict, currentPart):
                            return
                        else:
                            ptuPosDataFound += 1
                            currentPart = searchParts.Header

            if currentPart == searchParts.Header:
                if ((headerLineIndicator0 in lineArray[0]) and (timeFound==1)):
                    headerFound += 1
                    lineArrayLength=len(lineArray)
                    if lineArray[lineArrayLength -1] == '':

                        nofMeasDataFields = len(lineArray)-1
                    else:

                        nofMeasDataFields = len(lineArray)

                    parMappingDict = setParDict(lineArray,nofMeasDataFields)

                    currentPart = searchParts.PtuPosData

            # Header lookup

            # PTU position lookup
            elif currentPart == searchParts.PtuPosData:
                endOfEmptyLines = 0
                if not((len(lineArray) == 1) and (lineArray[0] == '')):
                    endOfEmptyLines = 1
                if endOfEmptyLines == 1:

                    lineArrayLength = len(lineArray)

                    if lineArray[lineArrayLength - 1] == '':

                        nofActualMeasDataFields = len(lineArray)-1
                    else:
                        nofActualMeasDataFields = len(lineArray)


                    if nofActualMeasDataFields != nofMeasDataFields :
                        txAntDiagParseData.errorList.append(strMeasName + ' incomplete or invalid ptu position measurement data')
                        return
                    else:
                        if not readPosPtuData(lineArray, txAntDiagParseData, nofMeasDataFields, parMappingDict, currentPart):
                            return
                        else:
                            ptuPosDataFound += 1

        if endOfEmptyLines == 0:
            txAntDiagParseData.errorList.append(strMeasName + ' could not find ptu position measurement data')
        # Setup error list
        if headerFound == 0:
            txAntDiagParseData.errorList.append(strMeasName + ' could not find measurement data header in file')
        if headerFound > 1:
            txAntDiagParseData.errorList.append(strMeasName + ' found header more than once in file, loaded meas. data probably incorrect')
        if maxHeaderFound == 0:
            txAntDiagParseData.errorList.append(strMeasName + ' could not find MAXIMUM(Spec) header in file')
        if maxHeaderFound > 1:
            txAntDiagParseData.errorList.append(strMeasName + '  MAXIMUM(Spec) found header more than once in file, loaded meas. data probably incorrect')


# Control wrapper for getMeasData
def readPosPtuData(lineArray, txAntDiagParseData, nofMeasDataFields, parMappingDict, currentPart):
    if not getMeasData(lineArray, txAntDiagParseData, nofMeasDataFields, parMappingDict, currentPart):
        if currentPart == searchParts.MaxHeaderData:
            txAntDiagParseData.errorList.append(str(txAntDiagParseData.fileNamePath) + ' Error - Reading MAXIMUM(SPEC) header measurement data')

        else:
            txAntDiagParseData.errorList.append(str(txAntDiagParseData.fileNamePath) + ' Error - Reading measurement data')
        return False
    return True

# Parses the measurements data part per one PTU position
def getMeasData(lineArray, txAntDiagParseData, nofMeasDataFields, parMappingDict, currentPart):
    bRet = True

    ptuPosMeasData = PtuPosMeasData()

    for key,par in parMappingDict.items():

        if par =='ptuAngleAz':
            temp = lineArray[key].replace(',', '.')

            if sp.checkDecimal(temp):
                ptuPosMeasData.d_PTU_AzAngle = float(temp)
            else:
                bRet = False

        elif par == 'ptuAngleEl':

            temp = lineArray[key].replace(',', '.')

            if sp.checkDecimal(temp):
                ptuPosMeasData.d_PTU_ElAngle = float(temp)
            else:
                bRet = False

        elif par == 'level':
            temp = lineArray[key]
            if sp.checkDecimal(temp):
                ptuPosMeasData.level_dBm = float(temp)
            else:
                bRet = False

        elif par == 'levelPowerMeter':
            temp = lineArray[key]
            if sp.checkDecimal(temp):
                ptuPosMeasData.levelPowMet_dBm = float(temp)
            else:
                bRet = False

        elif par == 'levelSpecMeas':

            temp = lineArray[key]
            if sp.checkDecimal(temp):
                ptuPosMeasData.levelMeasSpec_dBm = float(temp)
            else:
                bRet = False

        elif par == 'levelPowerMeterMeas':
            temp = lineArray[key]
            if sp.checkDecimal(temp):
                ptuPosMeasData.levelMeasPowMet_dBm = float(temp)
            else:
                bRet = False

        elif par == 'frequency':
            temp = lineArray[key]
            if sp.checkDecimal(temp):
                ptuPosMeasData.frequenz_MHz = float(temp)
            else:
                bRet = False

        elif par == 'time':
            temp = lineArray[key]
            if sp.checkDecimal(temp):
                ptuPosMeasData.time_ms = float(temp)
            else:
                bRet = False

    if not bRet:
        return False

    # Append the PTU position measurements per position to the ptuPosDataList
    if currentPart == searchParts.MaxHeaderData:
        txAntDiagParseData.ptuMaxSpecHeaderData = ptuPosMeasData
    else:
        txAntDiagParseData.ptuPosDataList.append(ptuPosMeasData)

    return bRet





import pytest
import sys
import os

# Add project root to path for imports
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

# Import Qt constants for proper comparison
from PySide6.QtWidgets import QFileDialog

def test_dialog_opens_user_cancels(mocker):
    """Test when user opens dialog but clicks Cancel."""
    
    # Use pytest mocker instead of unittest.mock
    mock_qt = mocker.patch('main.qt')
    
    # Mock the file dialog
    mock_qt.parseProtAddAndParsePtu2FDialog.exec_.return_value = False  # Dialog cancelled
    
    # Only need this for string concatenation
    mock_qt.parseProtSepParPtu2FilesStr = "Files: "
    
    # Create simple antenna system config mock matching real module structure
    class MockModule:
        pass
    mock_antSysDiag = MockModule()
    mock_antSysDiag.plotSetup = {'useSensorMuster': 0, 'sensorMusterList': []}
    mocker.patch('main.antSysDiag', mock_antSysDiag)
    
    # Mock clearTable separately to avoid MagicMock in range() operations
    mocker.patch('main.clearTable')
    
    # Import and call the function that gets triggered by button click
    from main import addAndParsePtu2Files
    
    # Create test data dictionaries
    sysDiagParsedDataDict = {'parsedDataList': []}
    separatePtu2ParsedDataDict = {'parsedDataList': []}
    separatePtu2FailedDataDict = {'parsedDataList': []}
    
    # Call the function (this simulates what happens when button is clicked)
    addAndParsePtu2Files(sysDiagParsedDataDict, separatePtu2ParsedDataDict, separatePtu2FailedDataDict)
    
    # Verify the dialog was opened
    mock_qt.parseProtAddAndParsePtu2FDialog.exec_.assert_called_once()


def test_dialog_opens_no_files_selected(mocker):
    """Test when user clicks OK but selects no files."""
    
    # SETUP MOCKS
    mock_qt = mocker.patch('main.qt')
    mock_qt.parseProtAddAndParsePtu2FDialog.exec_.return_value = True  # User clicked OK
    mock_qt.parseProtAddAndParsePtu2FDialog.selectedFiles.return_value = []  # But no files selected
    
    # Only need this for string concatenation
    mock_qt.parseProtSepParPtu2FilesStr = "Files: "
    
    # Create simple antenna system config mock matching real module structure
    class MockModule:
        pass
    mock_antSysDiag = MockModule()
    mock_antSysDiag.plotSetup = {'useSensorMuster': 0, 'sensorMusterList': []}
    mocker.patch('main.antSysDiag', mock_antSysDiag)
    
    # Mock clearTable separately to avoid MagicMock in range() operations
    mocker.patch('main.clearTable')
    
    # CALL FUNCTION
    from main import addAndParsePtu2Files
    sys_dict = {'parsedDataList': []}
    sep_parsed = {'parsedDataList': []}
    sep_failed = {'parsedDataList': []}
    
    addAndParsePtu2Files(sys_dict, sep_parsed, sep_failed)
    
    # VERIFY
    mock_qt.parseProtAddAndParsePtu2FDialog.exec_.assert_called_once()  # Dialog opened
    mock_qt.parseProtAddAndParsePtu2FDialog.selectedFiles.assert_called_once()  # Files requested (but empty list returned)
    assert len(sys_dict['parsedDataList']) == 0  # No data added
    assert len(sep_parsed['parsedDataList']) == 0
    assert len(sep_failed['parsedDataList']) == 0


def test_files_selected_all_parse_successfully(mocker):
    """Test when user selects real PTU files and all parse without errors."""
    
    # SETUP MOCKS - Only mock UI components
    mock_qt = mocker.patch('main.qt')
    mock_qt.parseProtAddAndParsePtu2FDialog.exec_.return_value = True  # User clicked OK
    
    # Use REAL PTU files with different characteristics - use absolute paths
    test_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    real_ptu_files = [
        os.path.join(test_dir, 'measurement_data', 'ptu_files', '2024-01-15_08-15-30_Tuner1_rf12345_dsp67890_Seq1_TxAnt0_Diagram0_Az_ptu2.txt'),
        os.path.join(test_dir, 'measurement_data', 'ptu_files', '2024-01-15_09-25-45_Tuner1_rf12345_dsp67890_Seq1_TxAnt1_Diagram2_Az_ptu2.txt'),
        os.path.join(test_dir, 'measurement_data', 'ptu_files', '2024-01-15_12-30-45_Tuner1_rf12345_dsp67890_Seq1_TxAnt0_Diagram1_El_ptu2.txt'),
        os.path.join(test_dir, 'measurement_data', 'ptu_files', '2024-01-15_14-45-10_Tuner1_rf12345_dsp67890_Seq1_TxAnt1_Diagram3_El_ptu2.txt')
    ]
    mock_qt.parseProtAddAndParsePtu2FDialog.selectedFiles.return_value = real_ptu_files
    
    # Mock only UI and logging components (let real parsing happen)
    mock_parseDataLog = mocker.patch('main.parseDataLog')
    mock_clearTable = mocker.patch('main.clearTable')
    mock_QTableWidgetItem = mocker.patch('main.QTableWidgetItem')
    
    # Create simple antenna system config mock matching real module structure
    # antSysDiag is a MODULE with plotSetup as a module-level dictionary
    class MockModule:
        pass
    mock_antSysDiag = MockModule()
    mock_antSysDiag.plotSetup = {'useSensorMuster': 0, 'sensorMusterList': []}
    mocker.patch('main.antSysDiag', mock_antSysDiag)
    
    # DON'T mock parsePtu2 - let real parsing happen!
    # DON'T mock parse data objects - they'll be real!
    
    # CALL FUNCTION  
    from main import addAndParsePtu2Files
    sys_dict = {'parsedDataList': []}
    sep_parsed = {'parsedDataList': []}
    sep_failed = {'parsedDataList': []}
    
    addAndParsePtu2Files(sys_dict, sep_parsed, sep_failed)
    
    # VERIFY
    mock_qt.parseProtAddAndParsePtu2FDialog.exec_.assert_called_once()
    mock_qt.parseProtAddAndParsePtu2FDialog.selectedFiles.assert_called_once()
    
    # With 4 real PTU files, all should parse successfully
    assert len(sys_dict['parsedDataList']) == 4  # All files added to main dict
    assert len(sep_parsed['parsedDataList']) == 4  # All files added to separate dict  
    assert len(sep_failed['parsedDataList']) == 0  # No failed files (assuming they're all valid)
    mock_clearTable.assert_called()  # Tables were cleared/updated
    mock_parseDataLog.info.assert_called()  # Files were logged
    assert mock_parseDataLog.info.call_count == 4  # One log call per file
    assert mock_QTableWidgetItem.call_count == 4  # One table item per successful file
    
    # Verify actual file paths are in the results (real parsing happened)
    file_paths = [data.fileTitle.fileNamePath for data in sys_dict['parsedDataList']]
    # Check that at least one of the real files made it through
    assert any('2024-01-15' in path for path in file_paths)


def test_files_selected_some_fail_parsing(mocker):
    """Test when user selects files but some fail to parse - using real good/bad files."""
    
    # SETUP MOCKS
    mock_qt = mocker.patch('main.qt')
    mock_qt.parseProtAddAndParsePtu2FDialog.exec_.return_value = True
    
    # Use one REAL good file and one REAL bad file - use absolute paths
    test_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    good_file = os.path.join(test_dir, 'measurement_data', 'ptu_files', '2024-01-15_08-15-30_Tuner1_rf12345_dsp67890_Seq1_TxAnt0_Diagram0_Az_ptu2.txt')
    bad_file = os.path.join(test_dir, 'tests', 'bad_ptu2_test_file.txt')
    
    mock_qt.parseProtAddAndParsePtu2FDialog.selectedFiles.return_value = [good_file, bad_file]
    
    # Mock UI and logging components only
    mock_parseDataLog = mocker.patch('main.parseDataLog')
    mock_clearTable = mocker.patch('main.clearTable')
    mock_QTableWidgetItem = mocker.patch('main.QTableWidgetItem')
    
    # Create simple antenna system config mock matching real module structure
    # antSysDiag is a MODULE with plotSetup as a module-level dictionary 
    class MockModule:
        pass
    mock_antSysDiag = MockModule()
    mock_antSysDiag.plotSetup = {'useSensorMuster': 0, 'sensorMusterList': []}
    mocker.patch('main.antSysDiag', mock_antSysDiag)
    
    # DON'T mock parsePtu2 - let real parsing happen with real files
    
    # CALL FUNCTION
    from main import addAndParsePtu2Files
    sys_dict = {'parsedDataList': []}
    sep_parsed = {'parsedDataList': []}
    sep_failed = {'parsedDataList': []}
    
    addAndParsePtu2Files(sys_dict, sep_parsed, sep_failed)
    
    # VERIFY - Mixed results: good file succeeds, bad file fails
    assert len(sys_dict['parsedDataList']) == 1  # Only good file in main dict
    assert len(sep_parsed['parsedDataList']) == 1  # Only good file in successful dict
    assert len(sep_failed['parsedDataList']) == 1  # Only bad file in failed dict
    
    # Check file paths to verify which file went where
    successful_path = sep_parsed['parsedDataList'][0].fileTitle.fileNamePath
    failed_path = sep_failed['parsedDataList'][0].fileTitle.fileNamePath
    
    assert '2024-01-15_08-15-30' in successful_path  # Good file succeeded  
    assert 'bad_ptu2_test_file.txt' in failed_path  # Bad file failed





def test_duplicate_files_not_added_twice(mocker):
    """Test that duplicate files aren't added to dictionaries twice - using real file."""
    
    # SETUP MOCKS
    mock_qt = mocker.patch('main.qt')
    mock_qt.parseProtAddAndParsePtu2FDialog.exec_.return_value = True
    
    # Use a real PTU file that will be selected as "duplicate" - use absolute path
    test_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    duplicate_file_path = os.path.join(test_dir, 'measurement_data', 'ptu_files', '2024-01-15_08-15-30_Tuner1_rf12345_dsp67890_Seq1_TxAnt0_Diagram0_Az_ptu2.txt')
    mock_qt.parseProtAddAndParsePtu2FDialog.selectedFiles.return_value = [duplicate_file_path]
    
    # Mock UI and logging components  
    mock_parseDataLog = mocker.patch('main.parseDataLog')
    mock_clearTable = mocker.patch('main.clearTable')
    mock_QTableWidgetItem = mocker.patch('main.QTableWidgetItem')
    
    # Create simple antenna system config mock matching real module structure
    # antSysDiag is a MODULE with plotSetup as a module-level dictionary
    class MockModule:
        pass 
    mock_antSysDiag = MockModule()
    mock_antSysDiag.plotSetup = {'useSensorMuster': 0, 'sensorMusterList': []}
    mocker.patch('main.antSysDiag', mock_antSysDiag)
    
    # DON'T mock parsePtu2 - let real parsing happen first time
    
    # FIRST CALL - Add file initially
    from main import addAndParsePtu2Files
    sys_dict = {'parsedDataList': []}
    sep_parsed = {'parsedDataList': []}
    sep_failed = {'parsedDataList': []}
    
    # First call adds the file
    addAndParsePtu2Files(sys_dict, sep_parsed, sep_failed)
    
    # Verify file was added initially
    assert len(sys_dict['parsedDataList']) == 1
    assert len(sep_parsed['parsedDataList']) == 1
    
    # SECOND CALL - Try to add same file again (simulate user selecting same file)
    addAndParsePtu2Files(sys_dict, sep_parsed, sep_failed)
    
    # VERIFY - File count should NOT increase (duplicate prevention worked)
    assert len(sys_dict['parsedDataList']) == 1  # Still only 1 file (not 2)
    assert len(sep_parsed['parsedDataList']) == 1  # Still only 1 file (not 2)  
    assert len(sep_failed['parsedDataList']) == 0  # Good file doesn't go to failed
    
    # Verify it's the correct file that was kept
    file_path = sys_dict['parsedDataList'][0].fileTitle.fileNamePath
    assert '2024-01-15_08-15-30' in file_path


def test_dialog_only_accepts_ptu2_files(mocker):
    """Test that file dialog is configured to filter for PTU2 .txt files."""
    
    # Mock the Qt GUI components completely
    mock_qt = mocker.patch('main.qt')
    
    # No need for MockDialog, use qt.parseProtAddAndParsePtu2FDialog directly
    mock_qt.parseProtAddAndParsePtu2FDialog.nameFilter.return_value = "PTU-2 Files (*_ptu2.txt)"
    mock_qt.parseProtAddAndParsePtu2FDialog.fileMode.return_value = QFileDialog.ExistingFiles
    mock_qt.parseProtAddAndParsePtu2FDialog.exec_.return_value = False  # User cancels dialog

    # Mock clearTable to avoid MagicMock in range() operations
    mocker.patch('main.clearTable')
    
    # VERIFY - Check the dialog filter configuration
    actual_filter = mock_qt.parseProtAddAndParsePtu2FDialog.nameFilter()
    
    # The dialog should have PTU-2 file filter set for .txt files
    assert "ptu2" in actual_filter.lower(), f"Expected PTU2 filter, got: {actual_filter}"
    assert ".txt" in actual_filter.lower(), f"Expected .txt extension, got: {actual_filter}"
    file_mode = mock_qt.parseProtAddAndParsePtu2FDialog.fileMode()
    assert file_mode == QFileDialog.ExistingFiles, "Dialog should allow multiple file selection"

    # Call the function and check dialog configuration after
    from main import addAndParsePtu2Files
    sys_dict = {'parsedDataList': []}
    sep_parsed = {'parsedDataList': []}
    sep_failed = {'parsedDataList': []}
    addAndParsePtu2Files(sys_dict, sep_parsed, sep_failed)
    # Check again after function call
    actual_filter_after = mock_qt.parseProtAddAndParsePtu2FDialog.nameFilter()
    file_mode_after = mock_qt.parseProtAddAndParsePtu2FDialog.fileMode()
    assert "ptu2" in actual_filter_after.lower(), f"Expected PTU2 filter after call, got: {actual_filter_after}"
    assert ".txt" in actual_filter_after.lower(), f"Expected .txt extension after call, got: {actual_filter_after}"
    assert file_mode_after == QFileDialog.ExistingFiles, "Dialog should allow multiple file selection after function call"


def test_counters_update_correctly(mocker):
    """Test that UI counters reflect correct parsed/failed file counts."""
    
    # SETUP MOCKS
    mock_qt = mocker.patch('main.qt')
    mock_qt.parseProtAddAndParsePtu2FDialog.exec_.return_value = True
    
    # Use mix of good and bad files - use absolute paths
    test_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    good_file = os.path.join(test_dir, 'measurement_data', 'ptu_files', '2024-01-15_08-15-30_Tuner1_rf12345_dsp67890_Seq1_TxAnt0_Diagram0_Az_ptu2.txt')
    bad_file = os.path.join(test_dir, 'tests', 'bad_ptu2_test_file.txt')
    mock_qt.parseProtAddAndParsePtu2FDialog.selectedFiles.return_value = [good_file, bad_file]
    
    # Mock the string used for concatenation
    mock_qt.parseProtSepParPtu2FilesStr = "Files: "
    
    # Mock UI components
    mock_parseDataLog = mocker.patch('main.parseDataLog')
    mock_clearTable = mocker.patch('main.clearTable')
    mock_QTableWidgetItem = mocker.patch('main.QTableWidgetItem')
    
    # Create simple antenna system config mock matching real module structure
    # antSysDiag is a MODULE with plotSetup as a module-level dictionary
    class MockModule:
        pass
    mock_antSysDiag = MockModule()
    mock_antSysDiag.plotSetup = {'useSensorMuster': 0, 'sensorMusterList': []}
    mocker.patch('main.antSysDiag', mock_antSysDiag)
    
    # CALL FUNCTION
    from main import addAndParsePtu2Files
    sys_dict = {'parsedDataList': []}
    sep_parsed = {'parsedDataList': []}
    sep_failed = {'parsedDataList': []}
    
    addAndParsePtu2Files(sys_dict, sep_parsed, sep_failed)
    
    # VERIFY - Counter label should be updated with correct count
    mock_qt.parseProtSepParPtu2FilesLabel.setText.assert_called()
    
    # Check that setText was called with the expected count (1 successful file)
    call_args = mock_qt.parseProtSepParPtu2FilesLabel.setText.call_args[0][0]
    assert '1' in call_args  # Should contain count of 1 successful file


def test_log_entries_created_with_timestamps_and_status(mocker):
    """Test that log entries are created for each file with proper content."""
    
    # SETUP MOCKS
    mock_qt = mocker.patch('main.qt')
    mock_qt.parseProtAddAndParsePtu2FDialog.exec_.return_value = True
    
    # Use absolute paths
    test_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    test_files = [
        os.path.join(test_dir, 'measurement_data', 'ptu_files', '2024-01-15_08-15-30_Tuner1_rf12345_dsp67890_Seq1_TxAnt0_Diagram0_Az_ptu2.txt'),
        os.path.join(test_dir, 'measurement_data', 'ptu_files', '2024-01-15_09-25-45_Tuner1_rf12345_dsp67890_Seq1_TxAnt1_Diagram2_Az_ptu2.txt')
    ]
    mock_qt.parseProtAddAndParsePtu2FDialog.selectedFiles.return_value = test_files
    
    # Mock UI components
    mock_parseDataLog = mocker.patch('main.parseDataLog')
    mock_clearTable = mocker.patch('main.clearTable')
    mock_QTableWidgetItem = mocker.patch('main.QTableWidgetItem')
    
    # Create simple antenna system config mock matching real module structure
    # antSysDiag is a MODULE with plotSetup as a module-level dictionary
    class MockModule:
        pass
    mock_antSysDiag = MockModule()
    mock_antSysDiag.plotSetup = {'useSensorMuster': 0, 'sensorMusterList': []}
    mocker.patch('main.antSysDiag', mock_antSysDiag)
    
    # CALL FUNCTION  
    from main import addAndParsePtu2Files
    sys_dict = {'parsedDataList': []}
    sep_parsed = {'parsedDataList': []}
    sep_failed = {'parsedDataList': []}
    
    # Reset mock to ensure clean state before our test
    mock_parseDataLog.info.reset_mock()
    
    addAndParsePtu2Files(sys_dict, sep_parsed, sep_failed)
    
    # VERIFY - Log entries created for each file
    # Check that exactly 2 log calls were made (one per file)
    assert mock_parseDataLog.info.call_count == 2, f"Expected 2 log entries, got {mock_parseDataLog.info.call_count}"
    
    # Verify log entries contain file paths (actual log content verification)
    logged_files = [call[0][0] for call in mock_parseDataLog.info.call_args_list]
    
    # Ensure both files were logged
    assert len(logged_files) == 2, f"Expected 2 logged files, got {len(logged_files)}"
    assert any('2024-01-15_08-15-30' in log_entry for log_entry in logged_files), "First file not found in logs"
    assert any('2024-01-15_09-25-45' in log_entry for log_entry in logged_files), "Second file not found in logs"



import numpy as np


def getAzPtuAngVals(txAntDiagParseData):
    angle = np.zeros(len(txAntDiagParseData.ptuPosDataList))
    for i in range(len(txAntDiagParseData.ptuPosDataList)):
        angle[i] = txAntDiagParseData.ptuPosDataList[i].d_PTU_AzAngle
    return angle

def getElPtuAngVals(txAntDiagParseData):
    angle = np.zeros(len(txAntDiagParseData.ptuPosDataList))
    for i in range(len(txAntDiagParseData.ptuPosDataList)):
        angle[i] = txAntDiagParseData.ptuPosDataList[i].d_PTU_ElAngle
    return angle

def getAzTarAngVals(txAntDiagParseData,upsideDown):
    if upsideDown == 0:
        angle = getAzPtuAngVals(txAntDiagParseData)*(-1)
    else:
        angle = getAzPtuAngVals(txAntDiagParseData)
    return angle

def getElTarAngVals(txAntDiagParseData, upsideDown):
    if upsideDown == 0:
        angle = getElPtuAngVals(txAntDiagParseData) * (-1)
    else:
        angle = getElPtuAngVals(txAntDiagParseData)
    return angle


def getLevel(txAntDiagParseData):
    level = np.zeros(getNofAngVals(txAntDiagParseData))
    for i in range(getNofAngVals(txAntDiagParseData)):
        level[i] = txAntDiagParseData.ptuPosDataList[i].level_dBm
    return level


def getLevelPowMet(txAntDiagParseData):
    level = np.zeros(getNofAngVals(txAntDiagParseData))
    for i in range(getNofAngVals(txAntDiagParseData)):
        level[i] = txAntDiagParseData.ptuPosDataList[i].levelPowMet_dBm
    return level


def getLevelMeasSpec(txAntDiagParseData):
    level = np.zeros(getNofAngVals(txAntDiagParseData))
    for i in range(getNofAngVals(txAntDiagParseData)):
        level[i] = txAntDiagParseData.ptuPosDataList[i].levelMeasSpec_dBm
    return level


def getLevelMeasPowMet(txAntDiagParseData):
    level = np.zeros(getNofAngVals(txAntDiagParseData))
    for i in range(getNofAngVals(txAntDiagParseData)):
        level[i] = txAntDiagParseData.ptuPosDataList[i].levelMeasPowMet_dBm
    return level


def getFrequency(txAntDiagParseData):
    freq = np.zeros(getNofAngVals(txAntDiagParseData))
    for i in range(getNofAngVals(txAntDiagParseData)):
        freq[i] = txAntDiagParseData.ptuPosDataList[i].frequenz_MHz
    return freq

def getTime(txAntDiagParseData):
    freq = np.zeros(getNofAngVals(txAntDiagParseData))
    for i in range(getNofAngVals(txAntDiagParseData)):
        freq[i] = txAntDiagParseData.ptuPosDataList[i].time_ms
    return freq


def getNofAngVals(ptu2ParseData):
     return len(ptu2ParseData.ptuPosDataList)




import os
from enum import Enum
import io
from shared import safeparse as sp

## Parse data module ##

# Initialize logger safely
try:
    import logconfig as log
    myLogger = log.mainLoadDataLogger.logger
except:
    myLogger = None

# Power level [dB] and Power level variance [dB^2] per measurement point(angle) inside  one chirp
class AntennaLevelData:
    def __init__(self):
        self.dLevel = -10000  # [dB]
        self.dVarianzLevel = -10000  # [dB ^ 2]

# Phase difference [°] and phase difference variance [dB^2] between two antennas per measurement point(angle) inside one chirp
class DiffPhaseData:
    def __init__(self):
        self.dDiffPhase = -10000 # [°]
        self.dVarianzDiffPhase = -10000 # [° ^ 2]

# Lists of AntennaLevelData and DiffPhaseData for one chirp, regarding more antennas
class ChirpMeasData:
    def __init__(self, antennaLevelData, diffPhaseData):
       self.antennaLevelDataList = antennaLevelData[:]
       self.diffPhaseDataList = diffPhaseData[:]

# PTU angle and chirp measurement data per angle
class PtuPosMeasData:
    def __init__(self):
        self.d_PTU_Angle = -10000 # [°]
        self.chirpMeasDataList = []
    # Pass a list of chirps
    def setChirpMeasDataList(self, chirpMeasDataList):
        self.chirpMeasDataList = chirpMeasDataList[:]

# Meeta data about chirp
class ChirpInfo:
    def __init__(self):
        self.chirp  = 0
        self.active = 0 # Chirp
        self.centerFrequency = 0 # [Hz]
        self.diffPhaseOffsetList=[]
    # Pass a list of phase difference offsets
    def setDiffPhases(self, diffPhaseOffsetList):
        self.diffPhaseOffsetList = []
        self.diffPhaseOffsetList = diffPhaseOffsetList[:]

# Meta data about antenna
class AntennaInfo:
    def __init__(self):
        self.rfSerial = None
        self.antennaType = 0
        self.nofAntennas = 0
        self.nofAntennaDiffValues = 0
        self.antennaDistancesList = []
    # Pass a list of antenna distances
    def setAntennaDistances(self, antennaDistancesList):
        self.antennaDistancesList = []
        self.antennaDistancesList = antennaDistancesList[:]
    # Get the number of differences between antennas
    def getNofAntennaDiffValues(self):
        return len(self.antennaDistancesList)
    # Set number of differences between antennas
    def setNofAntennaDiffValues(self, nofAntennaDiffValues):
        self.nofAntennaDiffValues = nofAntennaDiffValues

class MeasSetupInfo:
    def __init__(self):
        self.phyDoppGenDist          = 0
        self.doppGenHeightPtuZero    = 0
        self.rfBoardMidHeightPtuZero = 0
        self.sensorUpsideDown        = 0
        self.antennaDiagramType      = ''
        self.oppositeToPanAngle      = 0

        self.phyDoppGenDistFound = 0
        self.doppGenHeightPtuZeroFound    = 0
        self.rfBoardMidHeightPtuZeroFound = 0
        self.sensorUpsideDownFound   = 0
        self.antennaDiagramTypeFound = 0
        self.oppositeToPanAngleFound = 0

# Data from the parsed PTU2 file, containing antenna info, chirp info list, and meaurements data for each ptu position
class PTU2ParseData:
    MAX_CHIRP = 4
    def __init__(self):
        self.bDataValid     = False
        self.fileNamePath   = ''
        self.errorList      = []
        self.antennaInfo    = AntennaInfo()
        self.chirpInfoList  = []
        self.ptuPosDataList = [] # Messdaten an der jeweiligen ptu position
        self.fileTitle      = FileTitle()
        self.measSetupInfo  = MeasSetupInfo()
    # Call parse data from the PTU2ParseData object (parseData wrapper)
    def parseFile(self, fileNamePath, useSensorMuster, sensorMusterList):
        self.errorList = []
        self.fileNamePath = fileNamePath
        parseFile(fileNamePath, self)
        self.fileTitle.parseFileTitle(self.fileNamePath, useSensorMuster, sensorMusterList)

# Meta data class from the PTU2 file title
class FileTitle:
    def __init__(self):
        self.dateAndTime = ''
        self.date = ''
        self.time = ''
        self.sensor  = ''
        self.tuner   = ''
        self.txAnt   = 0
        self.seqNum  = 0
        self.diagram = 0
        self.rf  = ''
        self.dsp = ''
        self.fileNamePath = ''
        self.fileName = ''
        self.sensorMusterList = []
        self.useSensorMuster  = 0
        self.sensorMusterValueDict = dict()

        self.dateAndTimeFound = 0
        self.tunerFound = 0
        self.rfFound  = 0
        self.dspFound = 0
        self.txAntFound   = 0
        self.seqNumFound  = 0
        self.diagramFound = 0
        self.sensorFound = 0
        self.sensorMusterFoundDict = dict()


    # Parse the PTU2 file title
    def parseFileTitle(self, fileNamePath,useSensorMuster,sensorMusterList):

        self.fileNamePath = fileNamePath
        self.useSensorMuster  = useSensorMuster
        self.sensorMusterList = sensorMusterList

        if useSensorMuster == 1:
            if len(sensorMusterList) != 0:
                for musterItem in sensorMusterList:
                    dictKey = str(musterItem)
                    self.sensorMusterFoundDict[dictKey] = 0

                    self.sensorMusterValueDict[dictKey] = ''
            else:
                myLogger.error('Empty sensor muster list!')

        pathParts = os.path.split(fileNamePath)

        fileName = pathParts[1]
        self.fileName = fileName

        fileNameParts = fileName.split('_')

        for part in fileNameParts:
            if self.dateAndTimeFound == 0:
                if part.startswith('20') and part[4] == '-':
                    self.dateAndTime = part
                    self.dateAndTimeFound = 1
                    self.date = part[0:10]
                    self.time = part[11:19]
            if self.tunerFound == 0:
                if part.startswith('Tuner'):
                    self.tuner = part
                    self.tunerFound = 1
            if self.rfFound == 0:
                if part.startswith('rf'):
                    self.rf = part
                    self.rfFound = 1
            if self.dspFound == 0:
                if part.startswith('dsp'):
                    self.dsp = part
                    self.dspFound = 1
            if self.seqNumFound == 0:
                if 'Seq' in part:
                    # self.seqNumFound = 1
                    part = part.strip('Seq')
                    if 'TxAnt' not in part:
                        if (sp.checkInt(part)):
                            self.seqNum = int(part)
                    else:
                        a = part.replace('TxAnt','.')

                        if (sp.checkInt(a[0])):
                            self.seqNum = int(a[0])
                            self.seqNumFound = 1
                        else:
                            self.seqNumFound = 0
                        if (sp.checkInt(a[2])):
                            self.txAnt = int(a[2])
                            self.txAntFound = 1
                        else:
                            self.txAntFound = 0
            if self.txAntFound == 0:
                if part.startswith('TxAnt'):
                    part = part.strip('TxAnt')
                    if(sp.checkInt(part)):
                        self.txAnt = int(part)
                        self.txAntFound = 1
            if self.diagramFound == 0:
                if part.startswith('Diagram'):
                    part = part.strip('Diagram')
                    if (sp.checkInt(part)):
                        self.diagram = int(part)
                        self.diagramFound = 1

            if self.useSensorMuster == 1:
                if len(self.sensorMusterList) != 0:
                    for musterItem in self.sensorMusterList:
                        musterItem = str(musterItem)
                        if self.sensorMusterFoundDict[musterItem] == 0:
                            if part.startswith(musterItem):
                                self.sensorMusterValueDict[musterItem] = part
                                self.sensorMusterFoundDict[musterItem] = 1

        if self.sensorFound == 0:
            if self.useSensorMuster == 1:

                if len(self.sensorMusterList) != 0:
                    i = 0
                    self.sensorFound = 1
                    for key,val in self.sensorMusterFoundDict.items():
                        if self.sensorMusterFoundDict[key] == 1:
                            binder = '' if i == 0 else '_'
                            self.sensor+=binder+ self.sensorMusterValueDict[key]
                            i += 1
                        else:
                            self.sensorFound = 0

            else:
                if self.rfFound == 1:
                    binder = '_' if self.dspFound == 1 else ''
                    self.sensor = self.rf + binder + self.dsp
                    self.sensorFound = 1

# PTU2 file parts procesing info
class searchParts(Enum):
    MeasSetupData = 0
    Header = 1
    AntennaData = 2
    ChirpData  = 3
    PtuPosData = 4


# Control wrapper for interpretData
def parseFile(fileNamePath,ptu2ParseData):
    if myLogger:
        myLogger.info('Parsing: ' + fileNamePath)
    else:
        print('Parsing: ' + fileNamePath)
    ptu2ParseData.fileNamePath = fileNamePath
    interpretData(fileNamePath,ptu2ParseData)
    if len(ptu2ParseData.errorList) == 0:
        ptu2ParseData.bDataValid = True
    else:
        ptu2ParseData.bDataValid = False
    return ptu2ParseData

# Read and interpret the data for every part of the PTU2 file
def interpretData(fileNamePath,ptu2ParseData):
    strMeasName = ptu2ParseData.fileNamePath
    headerLineIndicator0 = 'PTU-Angle['

    measSetupLineIndicator0 = 'Gen21a_PhysicalDopplergenDistance[cm]:'
    measSetupLineIndicator1 = 'Gen21c_DopplerGenHeightToPtuZero[cm]:'
    measSetupLineIndicator2 = 'Gen21d_RFBoardMiddleHeightToPtuZero[cm]:'
    measSetupLineIndicator3 = 'Ptu02_PtuSensorUpsideDown'
    measSetupLineIndicator4 = 'AntennaDiagrammType[1]:'
    measSetupLineIndicator5 = 'OppositeToPanAngle[deg]:'

    # headerLineIndicator1 = 'Level-Ant0[dB]'

    currentPart = searchParts.MeasSetupData

    headerFound = 0
    antennaDataFound = 0
    chirpDataFound   = 0
    ptuPosDataFound  = 0
    chirpCounter = 0

    # Try to read the file
    try:
        f = io.open(fileNamePath, encoding='utf-8')
    except (IOError, UnicodeDecodeError) as err:
        try:
            f = io.open(fileNamePath, encoding='windows-1252')
        except IOError as ioErr:
            # errorString='try reading: ' +fileNamePath+ ' Exception = ' + str(ioErr)
            errorString = strMeasName + ' File does not exist! Insert a valid file path.'
            ptu2ParseData.errorList.append(errorString)
    else:
        # Read the lines of the file and remove all white spaces
        with f:
            content = f.readlines()
        content = [line.strip() for line in content]

        # Traverse through every line of the content
        for line in content:
            # Note: File; lines have been removed from PTU files directly
            # Measurement setup info lookup
            if measSetupLineIndicator0 in line:
                rest = line.strip(measSetupLineIndicator0)
                rest = rest.replace(',', '.')
                if sp.checkDecimal(rest):
                    ptu2ParseData.measSetupInfo.phyDoppGenDist = float(rest)
                    ptu2ParseData.measSetupInfo.phyDoppGenDistFound = 1
                else:
                    ptu2ParseData.measSetupInfo.phyDoppGenDistFound = 0
            elif measSetupLineIndicator1 in line:
                rest = line.strip(measSetupLineIndicator1)
                rest = rest.replace(',', '.')
                if sp.checkDecimal(rest):
                    ptu2ParseData.measSetupInfo.doppGenHeightPtuZero = float(rest)
                    ptu2ParseData.measSetupInfo.doppGenHeightPtuZeroFound = 1
                else:
                    ptu2ParseData.measSetupInfo.doppGenHeightPtuZeroFound = 0
            elif measSetupLineIndicator2 in line:
                rest = line.strip(measSetupLineIndicator2)
                rest = rest.replace(',', '.')
                if sp.checkDecimal(rest):
                    ptu2ParseData.measSetupInfo.rfBoardMidHeightPtuZero = float(rest)
                    ptu2ParseData.measSetupInfo.rfBoardMidHeightPtuZeroFound = 1
                else:
                    ptu2ParseData.measSetupInfo.rfBoardMidHeightPtuZeroFound = 0
            elif measSetupLineIndicator3 in line:
                l = len(line)
                c = line[l-1]
                if sp.checkInt(c):
                    ptu2ParseData.measSetupInfo.sensorUpsideDown = int(c)
                    ptu2ParseData.measSetupInfo.sensorUpsideDownFound = 1
                else:
                    ptu2ParseData.measSetupInfo.sensorUpsideDownFound = 0
            elif measSetupLineIndicator4 in line:
                rest = line.replace(measSetupLineIndicator4, '').strip()
                if ('Az' in rest) or ('El' in rest):
                    ptu2ParseData.measSetupInfo.antennaDiagramType = rest
                    ptu2ParseData.measSetupInfo.antennaDiagramTypeFound = 1
                else:
                    ptu2ParseData.measSetupInfo.antennaDiagramTypeFound = 0
            elif measSetupLineIndicator5 in line:
                rest = line.strip(measSetupLineIndicator5)
                rest = rest.replace(',', '.')
                if sp.checkInt(rest):
                    ptu2ParseData.measSetupInfo.oppositeToPanAngle = int(rest)
                    ptu2ParseData.measSetupInfo.oppositeToPanAngleFound = 1
                else:
                    ptu2ParseData.measSetupInfo.oppositeToPanAngleFound = 0

            # Swich to header lookup
            else:
                lineArray = line.split(';')
                if (headerLineIndicator0 in lineArray[0]):
                    headerFound += 1
                    currentPart = searchParts.Header

            # Header lookup
            if currentPart == searchParts.Header:
                if headerFound == 1:
                    currentPart = searchParts.AntennaData

            # Antenna info lookup
            elif currentPart == searchParts.AntennaData:
                if antennaDataFound == 0:
                    if len(lineArray) > 3:
                        antennaDataFound += 1
                if antennaDataFound > 0:
                    if not getAntennaData(lineArray, ptu2ParseData):
                        return
                    currentPart = searchParts.ChirpData
                    ptu2ParseData.chirpInfoList = []

            # Chirp info lookup
            elif currentPart == searchParts.ChirpData:
                if chirpCounter < PTU2ParseData.MAX_CHIRP:
                    if len(lineArray) == 3 + ptu2ParseData.antennaInfo.getNofAntennaDiffValues():
                        if not readChirpData(lineArray,ptu2ParseData,chirpCounter):
                            return
                        chirpCounter += 1
                if  chirpCounter == PTU2ParseData.MAX_CHIRP:
                    currentPart = searchParts.PtuPosData
                    ptu2ParseData.ptuPosDataList = []

            # PTU position lookup
            elif currentPart == searchParts.PtuPosData:
                if not readPosPtuData(lineArray,ptu2ParseData):
                    return
                else:
                    ptuPosDataFound += 1

        # Setup error list
        if headerFound == 0:
            ptu2ParseData.errorList.append(strMeasName + ' could not find header in file, no meas. data loaded')
        if headerFound > 1:
            ptu2ParseData.errorList.append(strMeasName + ' found header more than once in file, loaded meas. data probably incorrect')
        if antennaDataFound == 0:
            ptu2ParseData.errorList.append(strMeasName + ' could not find antenna data in file, no measdata loaded')
        if antennaDataFound > 1:
            ptu2ParseData.errorList.append(strMeasName + ' found antenna data more than once in file, loaded measdata probably incorrect')
        if chirpCounter != ptu2ParseData.MAX_CHIRP:
            ptu2ParseData.errorList.append(strMeasName + ' found (only) ' + str(chirpCounter)+ ' Chirp data of ' + str(ptu2ParseData.MAX_CHIRP) + ' Chirps')

# Parses the antenna info part
def getAntennaData(lineArray, ptu2ParseData):
    bRet = True
    if len(lineArray) >= 4:
        index = 0
        nofDiffValues=0

        temp = lineArray[index].replace(',', '.')
        if(sp.checkWholeNum(temp)):
            ptu2ParseData.antennaInfo.rfSerial = int(float(temp))
        else:
            bRet= False

        index += 1

        temp = lineArray[index].replace(',', '.')
        if(sp.checkWholeNum(temp)):
            ptu2ParseData.antennaInfo.antennaType = int(float(temp))
        else:
            bRet = False

        index += 1

        temp = lineArray[index].replace(',', '.')
        if(sp.checkWholeNum(temp)):
            ptu2ParseData.antennaInfo.nofAntennas = int(float(temp))
        else:
            bRet = False

        index += 1

        temp = lineArray[index].replace(',', '.')
        if(sp.checkWholeNum(temp)):
           nofDiffValues = int(float(temp))
        else:
            bRet = False

        index += 1

        ptu2ParseData.antennaInfo.setNofAntennaDiffValues(nofDiffValues)

        if len(lineArray) == (index + nofDiffValues):
            antennaDistList = []
            for antennaDistIndex in range(nofDiffValues):
                tempDistance = 0
                temp = lineArray[index+antennaDistIndex].replace(',','.')
                if (sp.checkDecimal(temp)):
                    tempDistance = float(temp)
                    tempDistance*= 0.01e-3 # [0, 01mm] = > [m]
                    antennaDistList.append(tempDistance)
                else:
                    bRet = False
            ptu2ParseData.antennaInfo.setAntennaDistances(antennaDistList)
        else:
            bRet = False
    else:
        bRet = False
    if bRet != True:
        errorString = ptu2ParseData.fileNamePath + ' Error 1 - Antennendaten einlesen'
        ptu2ParseData.errorList.append(errorString)
    return bRet

# Control wrapper for getChirpData
def readChirpData(lineArray,ptu2ParseData,chirpcounter):
    if not getChirpData(lineArray,ptu2ParseData):
        ptu2ParseData.errorList.append(str(ptu2ParseData.fileNamePath) + ' Error 2 - Chirp ' + str(chirpcounter) + ' daten einlesen')
        return False
    else:
        return True

# Parses the chirp info part
def getChirpData(lineArray, ptu2ParseDataClass):
    bRet = True
    if len(lineArray) != (3 + ptu2ParseDataClass.antennaInfo.getNofAntennaDiffValues()):
        return False
    chirpInfo = ChirpInfo()

    temp = lineArray[0].replace(',','.')
    if sp.checkDecimal(temp):
        chirpInfo.chirp = float(temp)
    else:
        bRet = False

    temp = lineArray[1].replace(',','.')
    if sp.checkDecimal(temp):
        chirpInfo.active = float(temp)
    else:
        bRet = False

    temp = lineArray[2].replace(',','.')
    if sp.checkDecimal(temp):
        chirpInfo.centerFrequency  = float(temp)
        chirpInfo.centerFrequency *= 1e4 # [10kHz] => [Hz]
    else:
        bRet = False

    diffPhaseOffsetList = []
    for index in range(ptu2ParseDataClass.antennaInfo.getNofAntennaDiffValues()):
        diffPhaseOffset = 0

        temp = lineArray[index + 3].replace(',','.')
        if sp.checkDecimal(temp):
            diffPhaseOffset = float(temp)
        else:
            bRet = False

        diffPhaseOffsetList.append(diffPhaseOffset)
    chirpInfo.setDiffPhases(diffPhaseOffsetList)
    ptu2ParseDataClass.chirpInfoList.append(chirpInfo)
    return bRet

# Control wrapper for getMeasData
def readPosPtuData(lineArray,ptu2ParseData):
    if not getMeasData(lineArray,ptu2ParseData):
        ptu2ParseData.errorList.append(str(ptu2ParseData.fileNamePath) + ' Error 3 - Messdaten einlesen')
        return False
    return True

# Parses the measurements data part per one PTU position
def getMeasData(lineArray,ptu2ParseData):
    bRet  =  True
    nofChirpValues = 2*(ptu2ParseData.antennaInfo.nofAntennas + ptu2ParseData.antennaInfo.getNofAntennaDiffValues())

    if len(lineArray) != (1+ ptu2ParseData.MAX_CHIRP * nofChirpValues):
        return False

    chirpMeasDataList = []

    ptuPosMeasData = PtuPosMeasData()
    # Traverse through each chirp and parse the PTU angle
    for chirpIndex in range(PTU2ParseData.MAX_CHIRP):

        temp = lineArray[0].replace(',','.')
        if sp.checkDecimal(temp):
            ptuPosMeasData.d_PTU_Angle = float(temp)
        else:
            bRet = False

        levelDataList = []
        # Parses the antenna level and variance for each antenna
        for antennaIndex in range(ptu2ParseData.antennaInfo.nofAntennas):
            levelData = AntennaLevelData()

            temp = lineArray[1+(nofChirpValues*chirpIndex) +2*antennaIndex].replace(',', '.')
            if sp.checkDecimal(temp):
                levelData.dLevel = float(temp)
            else:
                bRet = False

            temp = lineArray[2 + (nofChirpValues * chirpIndex) + 2 * antennaIndex].replace(',', '.')
            if sp.checkDecimal(temp):
                levelData.dVarianzLevel = float(temp)
            else:
                bRet = False
            levelDataList.append(levelData)

        diffPhaseDataList = []
        # Parses the phase difference and variance for each antenna
        for phaseIndex in range(ptu2ParseData.antennaInfo.getNofAntennaDiffValues()):
            diffPhaseData = DiffPhaseData()

            temp = lineArray[1+ (2*ptu2ParseData.antennaInfo.nofAntennas) + (chirpIndex*nofChirpValues) +(2*phaseIndex)].replace(',', '.')
            if sp.checkDecimal(temp):
                diffPhaseData.dDiffPhase = float(temp)
            else:
                bRet = False

            temp = lineArray[2+(2*ptu2ParseData.antennaInfo.nofAntennas) + (chirpIndex*nofChirpValues) +(2*phaseIndex)].replace(',', '.')
            if sp.checkDecimal(temp):
                diffPhaseData.dVarianzDiffPhase = float(temp)
            else:
                bRet = False
            diffPhaseDataList.append(diffPhaseData)
        if not bRet:
            return False
        # Packing the level and phase difference values to chirp measurements data (ChirpMeasData)
        chirpMeasData = ChirpMeasData(levelDataList,diffPhaseDataList)
        # Appending the chirp measurements data to a chirp list
        chirpMeasDataList.append(chirpMeasData)
    # Set the chirp measurements list to the PTU position data class (PtuPosMeasData)
    ptuPosMeasData.setChirpMeasDataList(chirpMeasDataList)
    # Append the PTU position measurements per position to the ptuPosDataList
    ptu2ParseData.ptuPosDataList.append(ptuPosMeasData)
    return bRet




import numpy as np

LIGHT_SPEED=299792458.0

def getAvgAntLevel(rxAnt,ptu2ParseData):
    nofActiveChirps=0
    avgData=np.zeros(getNofAngVals(ptu2ParseData))
    for chirp in range(ptu2ParseData.MAX_CHIRP):
        currData = getAntLevel(rxAnt, chirp, ptu2ParseData)
        if ptu2ParseData.chirpInfoList[chirp].active==1:
            nofActiveChirps+=1
            for i in range(getNofAngVals(ptu2ParseData)):
                currData[i] = 10 ** (currData[i] / 10) #To Watt
            avgData=avgData + currData
    avgData=avgData/nofActiveChirps
    avgData=10 * np.log10(avgData)
    return avgData

def getAvgNormAntLevel(rxAnt,ptu2ParseData):
    nofActiveChirps=0
    avgData=np.zeros(getNofAngVals(ptu2ParseData))
    for chirp in range(ptu2ParseData.MAX_CHIRP):
        currData = getNormAntLevel(rxAnt, chirp, ptu2ParseData)
        if ptu2ParseData.chirpInfoList[chirp].active==1:
            nofActiveChirps+=1
            for i in range(getNofAngVals(ptu2ParseData)):
                currData[i] = 10 ** (currData[i] / 10) #To Watt
            avgData=avgData + currData
    avgData=avgData/nofActiveChirps
    avgData=10 * np.log10(avgData)
    return avgData

# def toTargetAngle(ptu2ParseData):
#     for i in range(len(ptu2ParseData.ptuPosDataList)):
#         ptu2ParseData.ptuPosDataList[i].d_PTU_Angle *=-1
#     return ptu2ParseData

def getPtuAngVals(ptu2ParseData):
    angle = np.zeros(len(ptu2ParseData.ptuPosDataList))
    for i in range(len(ptu2ParseData.ptuPosDataList)):
        angle[i] = ptu2ParseData.ptuPosDataList[i].d_PTU_Angle
    return angle

def getTarAngVals(ptu2ParseData,upsideDown):
    if upsideDown==0:
        angle= getPtuAngVals(ptu2ParseData)*(-1)
    else:
        angle =getPtuAngVals(ptu2ParseData)
    return angle

def getAntLevel(rxAnt,chirp,ptu2ParseData):
    level=np.zeros(getNofAngVals(ptu2ParseData))
    for i in range(getNofAngVals(ptu2ParseData)):
        level[i]=ptu2ParseData.ptuPosDataList[i].chirpMeasDataList[chirp].antennaLevelDataList[rxAnt].dLevel
    return level

def getNormAntLevel(rxAnt,chirp,ptu2ParseData):
    normLevel=getAntLevel(rxAnt,chirp,ptu2ParseData)
    normLevel=normLevel - np.max(normLevel)
    return normLevel

def getNofAngVals(ptu2ParseData):
     return len(ptu2ParseData.ptuPosDataList)

def getPhaseDiff(phaseDiffIdx, chirp, ptu2ParseData):
    phaseDiff=np.zeros(getNofAngVals(ptu2ParseData))
    for i in range(getNofAngVals(ptu2ParseData)):
        phaseDiff[i]=ptu2ParseData.ptuPosDataList[i].chirpMeasDataList[chirp].diffPhaseDataList[phaseDiffIdx].dDiffPhase
    return phaseDiff

def getAvgPhaseDiff(phaseDiffIdx, ptu2ParseData):
    nofActiveChirps=0
    avgData=np.zeros(getNofAngVals(ptu2ParseData))
    for chirp in range(ptu2ParseData.MAX_CHIRP):
        # currData = getPhaseDiff(phaseDiffIdx, chirp, ptu2ParseData)
        currData = getPhaseDiffOffCorr(phaseDiffIdx, chirp, ptu2ParseData)
        if ptu2ParseData.chirpInfoList[chirp].active==1:
            nofActiveChirps+=1
            avgData=avgData + currData
    avgData=avgData/nofActiveChirps
    return avgData

def getAvgPhaseDiffDiff(phaseDiffIdx, ptu2ParseData):
    nofActiveChirps=0
    avgData=np.zeros(getNofAngVals(ptu2ParseData))
    for chirp in range(ptu2ParseData.MAX_CHIRP):
        currData = getPhaseDiffDiff(phaseDiffIdx, chirp, ptu2ParseData)
        if ptu2ParseData.chirpInfoList[chirp].active==1:
            nofActiveChirps+=1
            avgData=avgData + currData
    avgData=avgData/nofActiveChirps
    return avgData

def getIdealPhaseDiff(phaseDiffIdx, chirp, ptu2ParseData):
    angle=getPtuAngVals(ptu2ParseData)
    # angle=np.arange(-90,91)
    angle=angle*(-1)
    antDistance =ptu2ParseData.antennaInfo.antennaDistancesList[phaseDiffIdx]
    centerFrequency=ptu2ParseData.chirpInfoList[chirp].centerFrequency
    idealPhaseDiff =((np.sin(angle / 180.0 * np.pi) * 360.0 * antDistance * centerFrequency) / LIGHT_SPEED)
    idealPhaseDiff=getPhaseInRange(idealPhaseDiff)
    return idealPhaseDiff

def getPhaseDiffDiff(phaseDiffIdx, chirp, ptu2ParseData):
    # phaseDiff=getPhaseDiff(phaseDiffIdx, chirp, ptu2ParseData)
    phaseDiffOffCorr = getPhaseDiffOffCorr(phaseDiffIdx, chirp, ptu2ParseData)
    idealPhaseDiff=getIdealPhaseDiff(phaseDiffIdx, chirp, ptu2ParseData)
    diff=phaseDiffOffCorr-idealPhaseDiff
    diff=getPhaseInRange(diff)
    return diff

def getPhaseDiffOffCorr(phaseDiffIdx, chirp, ptu2ParseData):
    phaseDiff = getPhaseDiff(phaseDiffIdx, chirp, ptu2ParseData)
    offset = ptu2ParseData.chirpInfoList[chirp].diffPhaseOffsetList[phaseDiffIdx]
    offsetCorr = phaseDiff + offset
    offsetCorr=getPhaseInRange(offsetCorr)
    return offsetCorr

# replace the 180° jumps with continious data, PTU angle regarded
def getPhaseDiffOffCorrCont(phaseDiffIdx, chirp, ptu2ParseData):
    phaseDiffCorr=getPhaseDiffOffCorr(phaseDiffIdx, chirp, ptu2ParseData)

    # x=copy.copy(phaseDiffCorr)
    zeroDegIdxOdd=0
    zeroDegIdxUpper=0
    zeroDegIdxLower=0

    medianDifference=0

    tolerance=0
    HALF_TURN=180
    WHOLE_TURN=360

    if len(phaseDiffCorr) % 2 ==0:
        zeroDegIdxLower =int(len(phaseDiffCorr) / 2) -1
        zeroDegIdxUpper =int(len(phaseDiffCorr)/2)

    else:
        zeroDegIdxOdd = int((len(phaseDiffCorr) - 1) / 2)
        zeroDegIdxLower = zeroDegIdxUpper =  zeroDegIdxOdd

    diff=0
    incr=0
    # valst=phaseDiffCorr[145]
    # phaseDiffCorr[145::]=valst
    # phaseDiffCorr[158] = 0
    # phaseDiffCorr[159]=50
    # phaseDiffCorr[160] =-50
    outputArray=np.zeros(len(phaseDiffCorr))
    for idx in range(zeroDegIdxUpper,len(phaseDiffCorr)):
        current=phaseDiffCorr[idx]
        previous=phaseDiffCorr[idx - 1]
        diff=(phaseDiffCorr[idx] - phaseDiffCorr[idx - 1])
        if (diff > HALF_TURN):
            incr -= WHOLE_TURN

        if diff < (-1*HALF_TURN):
            incr += WHOLE_TURN

        outputArray[idx] = phaseDiffCorr[idx] + incr

    diff = 0
    incr = 0
    for idx in range(zeroDegIdxLower, -1, -1):
        current = phaseDiffCorr[idx]
        previous = phaseDiffCorr[idx + 1]
        diff = (phaseDiffCorr[idx] - phaseDiffCorr[idx + 1])
        if (diff > HALF_TURN):
            incr -= WHOLE_TURN

        if diff < (-1 * HALF_TURN):
            incr += WHOLE_TURN

        outputArray[idx] = phaseDiffCorr[idx] + incr

    return outputArray

    # posJumpRange=(HALF_TURN-tolerance,HALF_TURN+tolerance)
    # negJumpRange=(-1*HALF_TURN-tolerance,-1*HALF_TURN+tolerance)
    # # upper part
    # posJump=False
    # negJump=False
    # upperPartJumpIndices=[]
    # upperPartJumpIndices.append(zeroDegIdxUpper)
    # lastIdx=len(phaseDiffCorr)-1
    #
    # for idx in range(zeroDegIdxUpper,len(phaseDiffCorr)):
    #
    #     #pos jump
    #     if posJumpRange[0] <= (phaseDiffCorr[idx] - phaseDiffCorr[idx - 1]) <= posJumpRange[1]:
    #         if not negJump:
    #             posJump=True
    #             upperPartJumpIndices.append(idx)
    #
    #     #neg jump
    #     if negJumpRange[0] <= (phaseDiffCorr[idx] - phaseDiffCorr[idx - 1]) <= negJumpRange[1]:
    #         if not posJump:
    #             negJump=True
    #             upperPartJumpIndices.append(idx)
    #
    # upperPartJumpIndices.append(lastIdx)
    #
    # # temp=[]
    # # tempI=[]
    # # idice=[i for i in range(81)]
    #
    # if posJump:
    #     for idxOfIdx in range(len(upperPartJumpIndices)):
    #
    #         if idxOfIdx !=0:
    #             endIdx = upperPartJumpIndices[idxOfIdx]
    #             startIdx = upperPartJumpIndices[idxOfIdx - 1]
    #             rev = idxOfIdx - 1
    #             if idxOfIdx != len(upperPartJumpIndices)-1:
    #
    #                 phaseDiffCorr[startIdx:endIdx] = [value - rev * WHOLE_TURN for value in phaseDiffCorr[startIdx:endIdx]]
    #
    #             else:
    #
    #                 phaseDiffCorr[startIdx:endIdx+1] = [value - rev * WHOLE_TURN for value in phaseDiffCorr[startIdx:endIdx+1]]
    #
    #
    #             # temp1=phaseDiffCorr[startIdx:endIdx+1] = [value - rev * WHOLE_TURN for value in phaseDiffCorr[startIdx:endIdx+1]]
    #             # temp.append(temp1)
    #             # tempI1=idice[startIdx:endIdx+1] = [i for i in idice[startIdx:endIdx+1]]
    #             # tempI.append(tempI1)
    #
    #
    # elif negJump:
    #     for idxOfIdx in range(len(upperPartJumpIndices)):
    #
    #         if idxOfIdx !=0:
    #             endIdx = upperPartJumpIndices[idxOfIdx]
    #             startIdx = upperPartJumpIndices[idxOfIdx - 1]
    #             rev = idxOfIdx - 1
    #             if idxOfIdx != len(upperPartJumpIndices)-1:
    #
    #                 phaseDiffCorr[startIdx:endIdx] = [value + rev * WHOLE_TURN for value in phaseDiffCorr[startIdx:endIdx]]
    #
    #             else:
    #
    #                 phaseDiffCorr[startIdx:endIdx+1] = [value + rev * WHOLE_TURN for value in phaseDiffCorr[startIdx:endIdx+1]]
    #
    #
    # # lower part
    # posJump=False
    # negJump=False
    # lowerPartJumpIndices = []
    # lowerPartJumpIndices.append(zeroDegIdxLower)
    # lastIdx = 0
    #
    # for idx in range(zeroDegIdxLower,-1,-1):
    #
    #     # pos jump
    #     if posJumpRange[0] <= (phaseDiffCorr[idx] - phaseDiffCorr[idx + 1]) <= posJumpRange[1]:
    #         if not negJump:
    #             posJump = True
    #             lowerPartJumpIndices.append(idx)
    #
    #     # neg jump
    #     elif negJumpRange[0] <= (phaseDiffCorr[idx] - phaseDiffCorr[idx + 1]) <= negJumpRange[1]:
    #         if not posJump:
    #             negJump = True
    #             lowerPartJumpIndices.append(idx)
    #
    # lowerPartJumpIndices.append(lastIdx)
    #
    # if posJump:
    #     for idxOfIdx in range(len(lowerPartJumpIndices)):
    #
    #         if idxOfIdx !=0:
    #             endIdx = lowerPartJumpIndices[idxOfIdx]
    #             startIdx = lowerPartJumpIndices[idxOfIdx - 1]
    #             rev = idxOfIdx - 1
    #
    #             if idxOfIdx!= len(lowerPartJumpIndices)-1:
    #
    #                 phaseDiffCorr[startIdx:endIdx:-1] = [value - rev * WHOLE_TURN for value in phaseDiffCorr[startIdx:endIdx:-1]]
    #             else:
    #
    #                 phaseDiffCorr[startIdx::-1] = [value - rev * WHOLE_TURN for value in phaseDiffCorr[startIdx::-1]]
    #
    # elif negJump:
    #
    #     for idxOfIdx in range(len(lowerPartJumpIndices)):
    #
    #         if idxOfIdx !=0:
    #             endIdx = lowerPartJumpIndices[idxOfIdx]
    #             startIdx = lowerPartJumpIndices[idxOfIdx - 1]
    #             rev = idxOfIdx - 1
    #
    #             if idxOfIdx!= len(lowerPartJumpIndices)-1:
    #
    #                 phaseDiffCorr[startIdx:endIdx:-1] = [value + rev * WHOLE_TURN for value in phaseDiffCorr[startIdx:endIdx:-1]]
    #
    #                 # temp1 = phaseDiffCorr[startIdx:endIdx:-1] = [value - rev * WHOLE_TURN for value in
    #                 #                                              phaseDiffCorr[startIdx:endIdx:-1]]
    #                 # temp.append(temp1)
    #                 # tempI1 = idice[startIdx:endIdx:-1] = [i for i in idice[startIdx:endIdx:-1]]
    #                 # tempI.append(tempI1)
    #             else:
    #
    #                 phaseDiffCorr[startIdx::-1] = [value + rev * WHOLE_TURN for value in phaseDiffCorr[startIdx::-1]]
    #
    #                 # temp1 = phaseDiffCorr[startIdx::-1] = [value - rev * WHOLE_TURN for value in
    #                 #                                              phaseDiffCorr[startIdx::-1]]
    #                 # temp.append(temp1)
    #                 # tempI1 = idice[startIdx::-1] = [i for i in idice[startIdx::-1]]
    #                 # tempI.append(tempI1)
    #
    #
    # return phaseDiffCorr

def getPhaseInRange(graphArray):
    arrayLength=len(graphArray)
    # phaseInRange=np.zeros(arrayLength)
    for i in range(arrayLength):
        if graphArray[i]> 180:
            graphArray[i]=graphArray[i]-360
        if graphArray[i] <(-180):
           graphArray[i]=graphArray[i]+360
    return graphArray



import os
import protocolsTest as protocolfilter
import pyside_qtgui as qt
from shared import safeparse as sp
import logconfig
import protocolsTest as protocolFilter
# loadDataLogger=logconfig.loadDataLogger
parseDataLogger=logconfig.mainLoadDataLogger
parseDataLog=logconfig.mainLoadDataLogger.logger
logconfig.mainLoadDataLogger.addTableHandler(qt.mainLogTableHandler)
errorStatus, errorSting = logconfig.mainLoadDataLogger.addFileHandler(logconfig.mainLoadDataLogFilePath, 'w+')
if errorStatus ==1:
    parseDataLog.warning(errorSting)



protocolpathdict = {
    'protocolPathTypeDictList': [],

}
acceptedprotocolpathdict = {
    'emptyProtList': [],
    'errorProtList': [],
    'abbortCritProtList': [],
    'limitExceededProtList': []

}
filteredprotocolpathdict = {
    'protocolpathlist': [],
}

parsedprotdict = {
    'protocolPathTypeDictList': [],
}

prottobeparseddict = {
    'protocolPathTypeDictList': [],
}


def getprotcols(abspath, LatestProtocol, BurnInStart, EOLError, EOLAbbortCriterian, EOLMeasLimitsExceeded, allprotocolspathnameslist):
    busefileswitherror = EOLError
    busefilesabbortcriterion = EOLAbbortCriterian
    busefileswithlimitsexceeded = EOLMeasLimitsExceeded

    # erste infos ob fehler, abbruchkriterien, messgrenzen ueberschreitungen
    preparseinfoprotocolslist, emptyprotocolslist, errorprotocolslist, abbortcritprotocolslist, limitexceededprotocolslist = protocolfilter.ProtocolInfoClass.PreParseProtcols(
        allprotocolspathnameslist)
    # filtere liste, nur die gewuenschten protokolle in liste

    acceptedprotocolslist, filteredprotocolslist = protocolfilter.ProtocolInfoClass.FilterProtocols(
        preparseinfoprotocolslist, busefileswitherror, busefilesabbortcriterion, busefileswithlimitsexceeded)

    if LatestProtocol:
        acceptedprotocolslist = protocolFilter.ProtocolInfoClass.FilterNewProtocols(acceptedprotocolslist)


    accprotlist = []

    filteredprotocolslist = [
        filteredprotocolslist[i].PathName for i in range(len(filteredprotocolslist))]

    accerrorprotlist = []
    accabbortcritprotlist = []
    acclimitexceededprotlist = []

    for cpreparseinfo in acceptedprotocolslist:
        strpathonly, strfilenameonly = os.path.split(cpreparseinfo.PathName)
        stfilenamewithout_ptu = strfilenameonly.replace('Protocol_Chip', '').replace('Protocol', '').replace('protocol', '')
        strsplitfilenameandextension = os.path.splitext(stfilenamewithout_ptu)
        parseDataLog.info(cpreparseinfo.PathName)


        # if len(ptu2FilesListDict['ptu2FilesList'])!=0 or len(txAntDiagListDict['txAntDiagList'])!=0:


        if busefileswitherror and cpreparseinfo.bErrorOccured:
            accerrorprotlist.append(cpreparseinfo.PathName)

        if busefilesabbortcriterion and cpreparseinfo.bAbortCriterion:
            accabbortcritprotlist.append(cpreparseinfo.PathName)

        if busefileswithlimitsexceeded and cpreparseinfo.bLimitExceeded:
            acclimitexceededprotlist.append(cpreparseinfo.PathName)

        accprotlist.append(cpreparseinfo.PathName)

        # elif len(ptu2FilesListDict['ptu2FilesList'])!=0 and:
    return accprotlist, accerrorprotlist, accabbortcritprotlist, acclimitexceededprotlist, filteredprotocolslist

def importprotocolsdirpath(abspath, LatestProtocol, BurnInStart, EOLError, EOLAbbortCriterian, EOLMeasLimitsExceeded):
    protocoldirpath = abspath
    protocolfileslist = protocolfilter.ProtocolInfoClass.GetAllProtocolsList(
        protocoldirpath)

    for protFileIdx in range(len(protocolfileslist)):

        protocolfileslist[protFileIdx] = protocolfileslist[protFileIdx].replace("\\", "/")


    filterprotocoldata(abspath, LatestProtocol, BurnInStart, EOLError, EOLAbbortCriterian, EOLMeasLimitsExceeded, protocolfileslist)



def getprotpathtypedictlist(protocolPathList, BurnInStart):

    outputList = []

    for protocol in protocolPathList:
    # if qt.parseProtBurnInCalGroup.checkedId() == 0:
        typeMuster   = 'Gen26_UseBurnInMeas[1]'
        filterMuster = '(12a) BurnIn Start Check results:'
        tempDict = dict()
        tempDict['protocolPath'] = protocol
        tempDict['type'] = 'Unclear'
        # Filter the 'BurnIn At Start' protocols
        filterProt = False
        try:
            with open(protocol, 'r') as protocolFile:

                content = protocolFile.readlines()

                lineNr = 0

                for line in content:
                    lineNr += 1
                    line = line.strip()

                    if line.startswith(typeMuster):
                        line = line.replace(' ','')
                        linePartsList = line.split(';')
                        if linePartsList[0] == typeMuster:
                            tempVal = -1
                            
                            if sp.checkInt(linePartsList[1]):

                                tempVal = int(linePartsList[1])

                            else:
                                parseDataLog.error('Invalid value for protocol BurnIn/Calibration auto-recognition: '+linePartsList[1]+'.')

                            if tempVal == 0:

                                tempDict['type'] = 'Calibration'

                            elif tempVal == 3:

                                tempDict['type'] = 'BurnIn'

                            else:
                                tempDict['type'] = 'Unclear'

                        else:
                            tempDict['type'] = 'Unclear'
                    
                    if line == filterMuster:
                        filterProt = True
                        break
            
            # Do no add 'BurnIn At Start' protocols filter
            if not BurnInStart:
                if tempDict['type'] == 'BurnIn':
                    if not filterProt:
                        outputList.append(tempDict)
                else:
                    outputList.append(tempDict)
            # Add protocol files without any filter
            else:
                outputList.append(tempDict)

        except Exception as e:
            parseDataLog.error('Opening protocol file: ' + str(e) + '.')

    return outputList



def filterprotocoldata(abspath, LatestProtocol, BurnInStart, EOLError, EOLAbbortCriterian, EOLMeasLimitsExceeded, protocolfileslist):


    if len(protocolfileslist) == 0:
        return

    auxprotpathlist = []

    for protdict in protocolpathdict['protocolPathTypeDictList']:
        auxprotpathlist.append(protdict['protocolPath'])

    # protNotPresent=0
    protnonreplist = []
    for protocolfilepath in protocolfileslist:
        if protocolfilepath not in auxprotpathlist:
            protnonreplist.append(protocolfilepath)

    acceptedcritprotlist, accerrorprotlist, accabbortcritprotlist, acclimitexceededprotlist, filteredprotocollist = getprotcols(
        abspath, LatestProtocol, BurnInStart, EOLError, EOLAbbortCriterian, EOLMeasLimitsExceeded, protnonreplist)

    acceptedprotocolpathdict['errorProtList'] += accerrorprotlist
    acceptedprotocolpathdict['abbortCritProtList'] += accabbortcritprotlist
    acceptedprotocolpathdict['limitExceededProtList'] += acclimitexceededprotlist

    acceptedprotpathtypedictlist = getprotpathtypedictlist(
        acceptedcritprotlist, BurnInStart)


    protocolpathdict['protocolPathTypeDictList'] += acceptedprotpathtypedictlist

    auxprotpathlist = []

    for protdict in protocolpathdict['protocolPathTypeDictList']:
        auxprotpathlist.append(protdict['protocolPath'])

    for protocolfilepath in filteredprotocollist:
        if protocolfilepath not in filteredprotocolpathdict['protocolpathlist']:

            filteredprotocolpathdict['protocolpathlist'].append(
                protocolfilepath)

    filteredprotocolpathdict['protocolpathlist'] = [
        protocolfilepath for protocolfilepath in filteredprotocolpathdict['protocolpathlist'] if protocolfilepath not in auxprotpathlist]

    auxparsedprotpathlist = []

    for protdict in parsedprotdict['protocolPathTypeDictList']:
        auxparsedprotpathlist.append(protdict['protocolPath'])


# Class containing the loaded data, including the parsed PTU2 files list, error status data, and meta data
class loadprotocol:
    def __init__(self, abspath, LatestProtocol, BurnInStart, EOLError, EOLAbbortCriterian, EOLMeasLimitsExceeded):
        self.loaddata(abspath, LatestProtocol, BurnInStart, EOLError, EOLAbbortCriterian, EOLMeasLimitsExceeded)
        self.protocolpathdict = protocolpathdict

    # Load parsed data for a specific search path
    def loaddata(self, abspath, LatestProtocol, BurnInStart, EOLError, EOLAbbortCriterian, EOLMeasLimitsExceeded):
        clear()
        importprotocolsdirpath(abspath, LatestProtocol, BurnInStart, EOLError, EOLAbbortCriterian, EOLMeasLimitsExceeded)

def clear():
    global protocolpathdict
    protocolpathdict = {
        'protocolPathTypeDictList': [],

    }
    global acceptedprotocolpathdict
    acceptedprotocolpathdict = {
        'emptyProtList': [],
        'errorProtList': [],
        'abbortCritProtList': [],
        'limitExceededProtList': []

    }
    global filteredprotocolpathdict
    filteredprotocolpathdict = {
        'protocolpathlist': [],
    }
    global parsedprotdict
    parsedprotdict = {
        'protocolPathTypeDictList': [],
    }
    global prottobeparseddict
    prottobeparseddict = {
        'protocolPathTypeDictList': [],
    }

